# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function

from datetime import datetime

from django.core.management.base import BaseCommand

from balafon.Emailing.models import Emailing
from balafon.Emailing.utils import send_newsletter


class Command(BaseCommand):
    help = "send all emailing marked ready for sending"

    def add_arguments(self, parser):
        parser.add_argument('max_nb', type=int, default=20, nargs='?')

    def handle(self, max_nb, *args, **options):
        # look for emailing to be sent
        verbose = options.get('verbosity', 0)
        
        emailings = Emailing.objects.filter(
            status__in=(Emailing.STATUS_SCHEDULED, Emailing.STATUS_SENDING),
            scheduling_dt__lte=datetime.now()
        )
        total_sent = 0
        for emailing in emailings:
            emailing.status = Emailing.STATUS_SENDING
            emailing.save()
            
            nb_sent = send_newsletter(emailing, max_nb)
            
            if verbose:
                print(nb_sent, "emails sent for emailing", emailing.id)
            
            total_sent += nb_sent
            
            if emailing.send_to.count() == 0:
                if verbose:
                    print("emailing", emailing.id, "done")
                emailing.status = Emailing.STATUS_SENT
                emailing.save()
                
            if total_sent > max_nb:
                break  # stop sending if we reached the allowed number



# -*- coding: utf-8 -*-
"""django-modeltranslation configuration"""

from __future__ import unicode_literals

from modeltranslation.translator import translator, TranslationOptions

from .models import EntityType


class EntityTypeTranslationOptions(TranslationOptions):
    fields = ('name',)


translator.register(EntityType, EntityTypeTranslationOptions)



# -*- coding: utf-8 -*-
"""test mandrill webhooks"""

from __future__ import unicode_literals

import datetime

from model_mommy import mommy

from balafon.unit_tests import TestCase
from balafon.Crm import models
from balafon.Emailing.models import Emailing
from balafon.Emailing.utils import on_bounce


class BounceTestCase(TestCase):
    """test bounce behavior"""

    def test_soft_bounce(self):
        """soft bounce: it should trace it but keep contact subscribed"""
        contact = mommy.make(models.Contact, email="toto@toto.fr")
        subscription1 = mommy.make(models.Subscription, contact=contact, accept_subscription=True)
        mommy.make(models.Subscription, contact=contact, accept_subscription=True)

        emailing = mommy.make(Emailing, subscription_type=subscription1.subscription_type)

        self.assertEqual(models.Action.objects.count(), 0)

        email = contact.email
        description = "Just an error"
        on_bounce("soft_bounce", email, description, False, contact.uuid, emailing.id)

        self.assertEqual(models.Action.objects.count(), 1)
        action = models.Action.objects.all()[0]
        self.assertEqual(action.subject, "toto@toto.fr - soft_bounce: "+description)
        self.assertEqual(action.planned_date.date(), datetime.date.today())
        self.assertEqual(action.type.name, "bounce")

        self.assertEqual(action.contacts.count(), 1)
        self.assertEqual(action.entities.count(), 0)

        contact = action.contacts.all()[0]
        self.assertEqual(contact.email, email)
        self.assertEqual(contact.subscription_set.count(), 2)
        for subscription in contact.subscription_set.all():
            self.assertEqual(subscription.accept_subscription, True)

        self.assertEqual(emailing.soft_bounce.count(), 1)
        self.assertEqual(emailing.hard_bounce.count(), 0)
        self.assertEqual(emailing.unsub.count(), 0)
        self.assertEqual(emailing.spam.count(), 0)
        self.assertEqual(emailing.rejected.count(), 0)

    def test_bounce_types(self):
        """test different types of bounces"""
        contact = mommy.make(models.Contact, email="toto@toto.fr")
        subscription1 = mommy.make(models.Subscription, contact=contact, accept_subscription=True)
        mommy.make(models.Subscription, contact=contact, accept_subscription=True)

        emailing = mommy.make(Emailing, subscription_type=subscription1.subscription_type)

        self.assertEqual(models.Action.objects.count(), 0)

        event_types = ("soft_bounce", "hard_bounce", "rejected", "spam", "unsub")

        email = contact.email
        description = "Just an error"
        for event_type in event_types:
            on_bounce(event_type, email, description, False, contact.uuid, emailing.id)

        self.assertEqual(emailing.soft_bounce.count(), 1)
        self.assertEqual(emailing.hard_bounce.count(), 1)
        self.assertEqual(emailing.unsub.count(), 1)
        self.assertEqual(emailing.spam.count(), 1)
        self.assertEqual(emailing.rejected.count(), 1)

    def test_bounce_long_description(self):
        """test what happen if bounce description is too long"""

        contact = mommy.make(models.Contact, email="toto@toto.fr")
        subscription1 = mommy.make(models.Subscription, contact=contact, accept_subscription=True)

        emailing = mommy.make(Emailing, subscription_type=subscription1.subscription_type)

        self.assertEqual(models.Action.objects.count(), 0)

        email = contact.email
        description = "Just an error"*100
        on_bounce("soft_bounce", email, description, False, contact.uuid, emailing.id)

        self.assertEqual(models.Action.objects.count(), 1)
        action = models.Action.objects.all()[0]
        text = "toto@toto.fr - soft_bounce: "+description
        self.assertEqual(action.subject, text[:200])
        self.assertEqual(action.planned_date.date(), datetime.date.today())
        self.assertEqual(action.type.name, "bounce")

        self.assertEqual(emailing.soft_bounce.count(), 1)
        self.assertEqual(emailing.hard_bounce.count(), 0)
        self.assertEqual(emailing.unsub.count(), 0)
        self.assertEqual(emailing.spam.count(), 0)
        self.assertEqual(emailing.rejected.count(), 0)

    def test_hard_bounce(self):
        """test hard bounce: trace it and unsubscribe"""
        contact = mommy.make(models.Contact, email="toto@toto.fr")
        subscription1 = mommy.make(models.Subscription, contact=contact, accept_subscription=True)
        mommy.make(models.Subscription, contact=contact, accept_subscription=True)

        emailing = mommy.make(Emailing, subscription_type=subscription1.subscription_type)

        self.assertEqual(models.Action.objects.count(), 0)

        email = contact.email
        description = "Just an error"
        on_bounce("hard_bounce", email, description, True, contact.uuid, emailing.id)

        self.assertEqual(models.Action.objects.count(), 1)
        action = models.Action.objects.all()[0]
        self.assertEqual(action.subject, "toto@toto.fr - hard_bounce: "+description)
        self.assertEqual(action.planned_date.date(), datetime.date.today())
        self.assertEqual(action.type.name, "bounce")

        self.assertEqual(action.contacts.count(), 1)

        self.assertEqual(action.entities.count(), 0)

        contact = action.contacts.all()[0]
        self.assertEqual(contact.email, email)
        self.assertEqual(contact.subscription_set.count(), 2)
        for subscription in contact.subscription_set.all():
            self.assertEqual(subscription.accept_subscription, False)

        self.assertEqual(emailing.soft_bounce.count(), 0)
        self.assertEqual(emailing.hard_bounce.count(), 1)
        self.assertEqual(emailing.unsub.count(), 0)
        self.assertEqual(emailing.spam.count(), 0)
        self.assertEqual(emailing.rejected.count(), 0)

    def test_hard_bounce_no_contact(self):
        """test hard bounce and no contact is matching"""
        self.assertEqual(models.Action.objects.count(), 0)

        emailing = mommy.make(Emailing)

        email = "toto@toto.fr"
        description = "Just an error"
        on_bounce("hard_bounce", email, description, True, None, emailing.id)

        self.assertEqual(models.Action.objects.count(), 1)
        action = models.Action.objects.all()[0]
        self.assertEqual(action.subject, "toto@toto.fr - hard_bounce: "+description)
        self.assertEqual(action.planned_date.date(), datetime.date.today())
        self.assertEqual(action.type.name, "bounce")

        self.assertEqual(action.contacts.count(), 0)
        self.assertEqual(action.entities.count(), 0)

        self.assertEqual(emailing.soft_bounce.count(), 0)
        self.assertEqual(emailing.hard_bounce.count(), 0)
        self.assertEqual(emailing.unsub.count(), 0)
        self.assertEqual(emailing.spam.count(), 0)
        self.assertEqual(emailing.rejected.count(), 0)

    def test_hard_bounce_several_contacts(self):
        """hard bounce for several contacts"""

        contact = mommy.make(models.Contact, email="toto@toto.fr")
        mommy.make(models.Subscription, contact=contact, accept_subscription=True)
        mommy.make(models.Subscription, contact=contact, accept_subscription=True)

        entity = mommy.make(models.Entity, email=contact.email)
        default_contact = entity.default_contact
        subscription1 = mommy.make(models.Subscription, contact=default_contact, accept_subscription=True)
        mommy.make(models.Subscription, contact=default_contact, accept_subscription=True)

        emailing = mommy.make(Emailing, subscription_type=subscription1.subscription_type)

        self.assertEqual(models.Action.objects.count(), 0)

        email = contact.email
        description = "Just an error"
        on_bounce("hard_bounce", email, description, True, contact.uuid, emailing.id)

        self.assertEqual(models.Action.objects.count(), 1)
        action = models.Action.objects.all()[0]
        self.assertEqual(action.subject, "toto@toto.fr - hard_bounce: "+description)
        self.assertEqual(action.planned_date.date(), datetime.date.today())
        self.assertEqual(action.type.name, "bounce")

        self.assertEqual(action.contacts.count(), 1)
        self.assertEqual(action.entities.count(), 1)

        contact = models.Contact.objects.get(id=contact.id)
        self.assertEqual(contact.email, email)
        self.assertEqual(contact.subscription_set.count(), 2)
        for subscription in contact.subscription_set.all():
            self.assertEqual(subscription.accept_subscription, False)

        contact = models.Contact.objects.get(id=default_contact.id)
        self.assertEqual(contact.email, "")
        self.assertEqual(contact.subscription_set.count(), 2)
        for subscription in contact.subscription_set.all():
            self.assertEqual(subscription.accept_subscription, False)

        self.assertEqual(emailing.soft_bounce.count(), 0)
        self.assertEqual(emailing.hard_bounce.count(), 1)
        self.assertEqual(emailing.unsub.count(), 0)
        self.assertEqual(emailing.spam.count(), 0)
        self.assertEqual(emailing.rejected.count(), 0)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0003_actiontype_hide_contacts_buttons'),
    ]

    operations = [
        migrations.AddField(
            model_name='actiontype',
            name='mail_to_subject',
            field=models.CharField(default=b'', help_text='This would be used as subject when sending the action by email', max_length=100, verbose_name='Subject of email', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""unit testing"""

from __future__ import unicode_literals

from django.contrib.auth.models import User

from balafon.unit_tests import TestCase


class BaseTestCase(TestCase):
    """Base class for tests"""

    def setUp(self):
        """before each test"""
        super(BaseTestCase, self).setUp()

        self.user = User.objects.create(username="toto", is_staff=True)
        self.user.set_password("abc")
        self.user.save()
        self._login()

    def _login(self):
        """login"""
        self.client.login(username="toto", password="abc")



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext, ugettext_lazy as _

import floppyforms.__future__ as forms
from registration import get_version as get_registration_version

from balafon.Crm.models import Contact, EntityType
from balafon.Crm.forms import ModelFormWithCity
from balafon.Crm.widgets import CityAutoComplete
from balafon.Emailing.forms import SubscriptionTypeFormMixin
from balafon.Profile.models import ContactProfile
from balafon.settings import has_entity_on_registration_form, get_registration_accept_terms_of_use_link


class ProfileForm(ModelFormWithCity, SubscriptionTypeFormMixin):
    """A form for updating user profile"""
    city = forms.CharField(
        required=False,
        label=_('City'),
        widget=CityAutoComplete(attrs={'placeholder': _('Enter a city'), 'size': '80'})
    )

    class Media:
        css = {
            'all': ('css/base/jquery-ui-1.9.2.css',),
        }
        js = ('js/jquery-ui-1.9.2.js',)
    
    class Meta:
        model = Contact
        fields = (
            'gender', 'lastname', 'firstname', 'birth_date', 'email', 'phone', 'mobile',
            'address', 'address2', 'address3', 'zip_code', 'city', 'cedex', 'country',
            'accept_notifications',
        )

    def __init__(self, *args, **kwargs):
        super(ProfileForm, self).__init__(*args, **kwargs)
        self._add_subscription_types_field(contact=self.instance)


class EmailField(forms.EmailField):
    """This field avoid duplicate emails"""
    def clean(self, value):
        super(EmailField, self).clean(value)
        if User.objects.filter(email__iexact=value).count() > 0:
            raise forms.ValidationError(
                ugettext("This email is already registered. Use the 'forgot password' link on the login page")
            )
        return value


class UserRegistrationForm(ModelFormWithCity, SubscriptionTypeFormMixin):
    """A form for creating a new account on a website"""
    email = EmailField(required=True, label=_("Email"), widget=forms.TextInput())
    password1 = forms.CharField(required=True, widget=forms.PasswordInput(), label=_("Password"))
    password2 = forms.CharField(required=True, widget=forms.PasswordInput(), label=_("Repeat your password"))

    entity_type = forms.ChoiceField(required=False, widget=forms.Select())
    entity = forms.CharField(
         required=False,
         widget=forms.TextInput(attrs={'placeholder': _('Name of the entity')})
    )

    groups = forms.MultipleChoiceField(widget=forms.CheckboxSelectMultiple(), label='', required=False)

    accept_termofuse = forms.BooleanField(
        label=_('Accept terms of use'),
        help_text=_("Check for accepting the terms of use")
    )

    city = forms.CharField(
        required=False,
        label=_('City'),
        widget=CityAutoComplete(attrs={'placeholder': _('Enter a city'), 'size': '80'})
    )

    class Meta:
        model = ContactProfile
        fields = (
            'email', 'password1', 'password2', 'entity_type', 'entity', 'gender', 'firstname', 'lastname',
            'phone', 'mobile', 'address',
            'zip_code', 'city', 'cedex', 'country', 'groups', 'accept_termofuse'
        )

    def __init__(self, *args, **kwargs):
        super(UserRegistrationForm, self).__init__(*args, **kwargs)

        if 'gender' in self.fields:
            # do not display Mrs and Mr
            self.fields['gender'].choices = ((0, '----------'), ) + Contact.GENDER_CHOICE[:2]

        if 'entity_type' in self.fields:
            self.fields['entity_type'].choices = [(0, ugettext('Individual'))]+[
                (et.id, et.name) for et in EntityType.objects.filter(subscribe_form=True)
            ]
        if not has_entity_on_registration_form():
            self.fields['entity_type'].inital = 0
            self.fields['entity_type'].widget = forms.HiddenInput()
            self.fields['entity'].widget = forms.HiddenInput()

        termsofuse_url = get_registration_accept_terms_of_use_link()
        if 'accept_termofuse' in self.fields and termsofuse_url:
            self.fields['accept_termofuse'].label = mark_safe(
                ugettext('Accept <a href="{0}">terms of use</a>').format(
                    termsofuse_url
                )
            )
        self._add_subscription_types_field()

    def clean_entity(self, ):
        entity_type = self.cleaned_data.get('entity_type', None)
        entity = self.cleaned_data['entity']
        if entity_type:
            if not entity:
                raise ValidationError(_("{0}: Please enter a name".format(entity_type)))
        return entity
        
    def clean_entity_type(self):
        try:
            entity_type_id = int(self.cleaned_data['entity_type'] or 0)
        except ValueError:
            raise ValidationError(ugettext('Invalid entity type'))
       
        if entity_type_id:
            try:
                return EntityType.objects.get(id=entity_type_id)
            except EntityType.DoesNotExist:
                raise ValidationError(ugettext('Unknown entity type'))
        
        return None

    def clean(self, *args, **kwargs):
        password1 = self.cleaned_data.get('password1', "")
        password2 = self.cleaned_data.get('password2', "")
        if password1 and (password1 != password2):
            raise forms.ValidationError(ugettext('Passwords are not the same'))
        return super(UserRegistrationForm, self).clean(*args, **kwargs)

    def save(self, commit=True):

        if get_registration_version() >= "2.0.0":
            # Django registration 2.0
            # The registration form should return a user

            email = self.cleaned_data["email"]
            username = email[:30]

            user = User.objects.create(
                username=username, email=email, is_active=False
            )
            password = self.cleaned_data.get('password1', "")
            user.set_password(password)

            return user
        else:
            # Django registration 1.0
            return super(UserRegistrationForm, self).save(commit)


class MinimalUserRegistrationForm(UserRegistrationForm):
    """A form for creating a new account on a website. Keep it minimal"""
    class Meta:
        model = ContactProfile
        fields = (
            'email', 'password1', 'password2', 'firstname', 'lastname', 'accept_termofuse',
            'zip_code', 'city', 'country', 'groups_ids',
        )

    def __init__(self, *args, **kwargs):
        super(MinimalUserRegistrationForm, self).__init__(*args, **kwargs)
        for hidden_field in ('entity', 'entity_type', 'zip_code', 'city', 'country', 'groups_ids'):
            self.fields[hidden_field].widget = forms.HiddenInput()

        for required_field in ('firstname', 'lastname'):
            self.fields[required_field].required = True


class MessageForm(forms.Form):
    """A form for sending a message to web site owner"""
    message = forms.CharField(
        required=True,
        widget=forms.Textarea(attrs={'placeholder': _("Your message"),})
    )
    
    def clean_message(self):
        message = self.cleaned_data['message']
        if len(message) > 10000:
            raise forms.ValidationError(ugettext('Message is too long'))
        return message



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.contrib.auth.backends import ModelBackend

from coop_cms.perms_backends import ArticlePermissionBackend

from balafon.Profile.utils import check_category_permission


class EmailModelBackend(ModelBackend):
    def authenticate(self, username=None, password=None, email=None):
        email = email or username
        kwargs = {
            'is_active': True,
            'email': email.strip(),
        }

        try:
            user = User.objects.get(**kwargs)
            if user.check_password(password):
                return user
            
        except User.DoesNotExist:
            return None

        except User.MultipleObjectsReturned:
            for next_user in User.objects.filter(**kwargs):
                if next_user.check_password(password):
                    return next_user
            return None


class ArticleCategoryPermissionBackend(ArticlePermissionBackend):
    
    def has_perm(self, user_obj, perm, obj=None):
        if obj:
            if check_category_permission(obj, perm, user_obj):
                return super(ArticleCategoryPermissionBackend, self).has_perm(user_obj, perm, obj)
        return False



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django import template


register = template.Library()


@register.filter
def dt_from(dt):
    try:
        return dt.split(" ")[0]
    except:
        return ""


@register.filter
def dt_to(dt):
    try:
        return dt.split(" ")[1]
    except:
        return ""



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-06 09:56
from __future__ import unicode_literals

from decimal import Decimal
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0013_auto_20180409_1301'),
    ]

    operations = [
        migrations.AddField(
            model_name='saleitem',
            name='percentage',
            field=models.DecimalField(decimal_places=2, default=Decimal('100'), help_text='Only a percentage of the item price', max_digits=5, verbose_name='percentage'),
        ),
    ]



# -*- coding: utf-8 -*-
"""unit testing"""

from __future__ import unicode_literals

from datetime import date, datetime
from decimal import Decimal
import xlrd

from django.contrib.auth.models import User
from django.core.urlresolvers import reverse

from colorbox.utils import assert_popup_redirects
from model_mommy import mommy

from balafon.unit_tests import TestCase
from balafon.Crm.models import ActionType, Action, Contact, Entity, ActionStatus
from balafon.Store import models


class CatalogueTest(TestCase):
    """It should export """

    def setUp(self):
        """before each test"""
        super(CatalogueTest, self).setUp()
        user = mommy.make(User, is_active=True, is_staff=True, is_superuser=False)
        user.set_password('abc')
        user.save()
        self.client.login(username=user.username, password='abc')
        self.user = user

    def test_export_all(self):
        """It should export all in xls"""
        mommy.make(models.StoreItem, available=True)
        mommy.make(models.StoreItem, available=True)

        response = self.client.get(reverse('store_xls_catalogue', args=[0]))

        self.assertEqual(response.status_code, 200)
        workbook = xlrd.open_workbook(file_contents=response.content, formatting_info=True)
        sheet = workbook.sheet_by_index(0)

        self.assertEqual(sheet.nrows, 1 + 2)  # header + 2 lines

    def test_export_all_available(self):
        """It should export only available items"""
        item1 = mommy.make(models.StoreItem, available=True)
        mommy.make(models.StoreItem, available=False)

        response = self.client.get(reverse('store_xls_catalogue', args=[0]))

        self.assertEqual(response.status_code, 200)
        workbook = xlrd.open_workbook(file_contents=response.content, formatting_info=True)
        sheet = workbook.sheet_by_index(0)

        self.assertEqual(sheet.nrows, 1 + 1)
        self.assertEqual(sheet.cell(1, 0).value, item1.fullname())

    def test_export_anonymous(self):
        """It should raise 403"""
        self.client.logout()

        item1 = mommy.make(models.StoreItem, available=True)
        mommy.make(models.StoreItem, available=False)

        response = self.client.get(reverse('store_xls_catalogue', args=[0]))

        self.assertEqual(response.status_code, 403)

    def test_export_category(self):
        """It should export only items of category"""
        category1 = mommy.make(models.StoreItemCategory)
        category2 = mommy.make(models.StoreItemCategory)

        item1 = mommy.make(models.StoreItem, available=True, category=category1)
        mommy.make(models.StoreItem, available=True, category=category2)

        response = self.client.get(reverse('store_xls_catalogue', args=[category1.id]))

        self.assertEqual(response.status_code, 200)
        workbook = xlrd.open_workbook(file_contents=response.content, formatting_info=True)
        sheet = workbook.sheet_by_index(0)

        self.assertEqual(sheet.nrows, 1 + 1)
        self.assertEqual(sheet.cell(1, 0).value, item1.fullname())


class ActionSummaryTest(TestCase):
    """It should export """

    def setUp(self):
        """before each test"""
        super(ActionSummaryTest, self).setUp()
        user = mommy.make(User, is_active=True, is_staff=True, is_superuser=False)
        user.set_password('abc')
        user.save()
        self.client.login(username=user.username, password='abc')
        self.user = user

    def test_view_export(self):
        """It should export all in xls"""
        action_type = mommy.make(ActionType, name="Bills of Month")

        mommy.make(models.StoreManagementActionType, action_type=action_type)

        action = mommy.make(Action, type=action_type, planned_date=date.today())
        vat_rate = mommy.make(models.VatRate, rate=Decimal(10))
        sale_item = mommy.make(models.SaleItem, sale=action.sale, vat_rate=vat_rate, pre_tax_price=Decimal(10))

        url = reverse('store_actions_summary', args=[action_type.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<form')

    def test_view_export_anonymous(self):
        """It should export all in xls"""
        self.client.logout()
        action_type = mommy.make(ActionType, name="Bills of Month")

        mommy.make(models.StoreManagementActionType, action_type=action_type)

        action = mommy.make(Action, type=action_type, planned_date=date.today())
        vat_rate = mommy.make(models.VatRate, rate=Decimal(10))
        sale_item = mommy.make(models.SaleItem, sale=action.sale, vat_rate=vat_rate, pre_tax_price=Decimal(10))

        url = reverse('store_actions_summary', args=[action_type.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 302)

    def test_export_redirect(self):
        """It should export all in xls"""
        action_type = mommy.make(ActionType, name="Bills of Month")

        mommy.make(models.StoreManagementActionType, action_type=action_type)

        action = mommy.make(Action, type=action_type, planned_date=datetime(2018, 6, 21))
        vat_rate = mommy.make(models.VatRate, rate=Decimal(10))
        sale_item = mommy.make(models.SaleItem, sale=action.sale, vat_rate=vat_rate, pre_tax_price=Decimal(10))

        data = {
            'start_date': date(2018, 6, 1),
            'end_date': date(2018, 6, 30),
        }

        url = reverse('store_actions_summary', args=[action_type.id])
        response = self.client.post(url, data)

        redirect_url = reverse('store_actions_summary_xls', args=[action_type.id, '2018-06-01', '2018-06-30'])

        assert_popup_redirects(response, redirect_url)

    def test_export_anonymous(self):
        """It should export all in xls"""

        self.client.logout()
        action_type = mommy.make(ActionType, name="Bills of Month", number_auto_generated=1)
        action_status = mommy.make(ActionStatus, name="Created")
        action_status = mommy.make(ActionStatus, name="Paid")

        mommy.make(models.StoreManagementActionType, action_type=action_type)

        contact = mommy.make(Contact, firstname="Pierre", lastname="Dupond")

        planned_date = datetime(2018, 6, 21)
        action = mommy.make(Action, type=action_type, planned_date=planned_date, status=action_status)
        action.contacts.add(contact)
        action.save()

        vat_rate = mommy.make(models.VatRate, rate=Decimal(10))
        sale_item = mommy.make(
            models.SaleItem, sale=action.sale, vat_rate=vat_rate, pre_tax_price=Decimal(10), quantity=1
        )

        url = reverse('store_actions_summary_xls', args=[action_type.id, '2018-06-01', '2018-06-30'])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 403)

    def test_export_one_for_contact(self):
        """It should export all in xls"""
        action_type = mommy.make(ActionType, name="Bills of Month", number_auto_generated=1)
        action_status = mommy.make(ActionStatus, name="Created")
        action_status = mommy.make(ActionStatus, name="Paid")

        mommy.make(models.StoreManagementActionType, action_type=action_type)

        contact = mommy.make(Contact, firstname="Pierre", lastname="Dupond")

        planned_date = datetime(2018, 6, 21)
        action = mommy.make(Action, type=action_type, planned_date=planned_date, status=action_status)
        action.contacts.add(contact)
        action.save()

        vat_rate = mommy.make(models.VatRate, rate=Decimal(10))
        sale_item = mommy.make(
            models.SaleItem, sale=action.sale, vat_rate=vat_rate, pre_tax_price=Decimal(10), quantity=1
        )

        url = reverse('store_actions_summary_xls', args=[action_type.id, '2018-06-01', '2018-06-30'])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        workbook = xlrd.open_workbook(file_contents=response.content, formatting_info=True)
        sheet = workbook.sheet_by_index(0)

        self.assertEqual(sheet.nrows, 2 + 2)

        values = [sheet.cell(1, col).value for col in range(sheet.ncols)]
        self.assertEqual(values[0], 1)
        self.assertEqual(values[1], "Pierre Dupond")
        self.assertEqual(datetime(*xlrd.xldate_as_tuple(values[2], workbook.datemode)).date(), planned_date.date())
        self.assertEqual(values[3], action_status.name)
        self.assertEqual(values[4], '')
        self.assertEqual(values[5], Decimal(10))
        self.assertEqual(values[6], Decimal(1))
        self.assertEqual(values[7], Decimal(11))

    def test_export_one_for_contact_before_start(self):
        """It should export all in xls"""
        action_type = mommy.make(ActionType, name="Bills of Month", number_auto_generated=1)
        action_status = mommy.make(ActionStatus, name="Created")
        action_status = mommy.make(ActionStatus, name="Paid")

        mommy.make(models.StoreManagementActionType, action_type=action_type)

        contact = mommy.make(Contact, firstname="Pierre", lastname="Dupond")

        planned_date = datetime(2018, 5, 31)
        action = mommy.make(Action, type=action_type, planned_date=planned_date, status=action_status)
        action.contacts.add(contact)
        action.save()

        vat_rate = mommy.make(models.VatRate, rate=Decimal(10))
        sale_item = mommy.make(
            models.SaleItem, sale=action.sale, vat_rate=vat_rate, pre_tax_price=Decimal(10), quantity=1
        )

        url = reverse('store_actions_summary_xls', args=[action_type.id, '2018-06-01', '2018-06-30'])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        workbook = xlrd.open_workbook(file_contents=response.content, formatting_info=True)
        sheet = workbook.sheet_by_index(0)

        self.assertEqual(sheet.nrows, 1)

        # Check emptiness
        for row in range(1, sheet.nrows):
            values = [sheet.cell(row, col).value for col in range(sheet.ncols)]
            self.assertEqual([""] * sheet.ncols, values)

    def test_export_one_for_contact_after_end(self):
        """It should export all in xls"""
        action_type = mommy.make(ActionType, name="Bills of Month", number_auto_generated=1)
        action_status = mommy.make(ActionStatus, name="Created")
        action_status = mommy.make(ActionStatus, name="Paid")

        mommy.make(models.StoreManagementActionType, action_type=action_type)

        contact = mommy.make(Contact, firstname="Pierre", lastname="Dupond")

        planned_date = datetime(2018, 7, 1)
        action = mommy.make(Action, type=action_type, planned_date=planned_date, status=action_status)
        action.contacts.add(contact)
        action.save()

        vat_rate = mommy.make(models.VatRate, rate=Decimal(10))
        sale_item = mommy.make(
            models.SaleItem, sale=action.sale, vat_rate=vat_rate, pre_tax_price=Decimal(10), quantity=1
        )

        url = reverse('store_actions_summary_xls', args=[action_type.id, '2018-06-01', '2018-06-30'])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        workbook = xlrd.open_workbook(file_contents=response.content, formatting_info=True)
        sheet = workbook.sheet_by_index(0)

        self.assertEqual(sheet.nrows, 1)

        # Check emptiness
        for row in range(1, sheet.nrows):
            values = [sheet.cell(row, col).value for col in range(sheet.ncols)]
            self.assertEqual([""] * sheet.ncols, values)

    def test_export_one_for_contact_other_type(self):
        """It should export all in xls"""
        action_type1 = mommy.make(ActionType, name="Bills of Month", number_auto_generated=1)
        action_type2 = mommy.make(ActionType, name="Bills of Month", number_auto_generated=1)
        action_status = mommy.make(ActionStatus, name="Created")
        action_status = mommy.make(ActionStatus, name="Paid")

        mommy.make(models.StoreManagementActionType, action_type=action_type2)

        contact = mommy.make(Contact, firstname="Pierre", lastname="Dupond")

        planned_date = datetime(2018, 6, 21)
        action = mommy.make(Action, type=action_type2, planned_date=planned_date, status=action_status)
        action.contacts.add(contact)
        action.save()

        vat_rate = mommy.make(models.VatRate, rate=Decimal(10))
        sale_item = mommy.make(
            models.SaleItem, sale=action.sale, vat_rate=vat_rate, pre_tax_price=Decimal(10), quantity=1
        )

        url = reverse('store_actions_summary_xls', args=[action_type1.id, '2018-06-01', '2018-06-30'])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        workbook = xlrd.open_workbook(file_contents=response.content, formatting_info=True)
        sheet = workbook.sheet_by_index(0)

        self.assertEqual(sheet.nrows, 1)

        # Check emptiness
        for row in range(1, sheet.nrows):
            values = [sheet.cell(row, col).value for col in range(sheet.ncols)]
            self.assertEqual([""] * sheet.ncols, values)

    def test_export_several_for_contacts(self):
        """It should export all in xls"""
        action_type = mommy.make(ActionType, name="Bills of Month", number_auto_generated=1)
        action_status1 = mommy.make(ActionStatus, name="Created")
        action_status2 = mommy.make(ActionStatus, name="Paid")

        mommy.make(models.StoreManagementActionType, action_type=action_type)

        contact1 = mommy.make(Contact, firstname="Pierre", lastname="Dupond")
        contact2 = mommy.make(Contact, firstname="Paul", lastname="Dupuis")

        planned_date1 = datetime(2018, 6, 1)
        action1 = mommy.make(Action, type=action_type, planned_date=planned_date1, status=action_status1)
        action1.contacts.add(contact1)
        action1.save()

        planned_date2 = datetime(2018, 6, 30, 18, 0)
        action2 = mommy.make(Action, type=action_type, planned_date=planned_date2, status=action_status2)
        action2.contacts.add(contact2)
        action2.save()

        vat_rate1 = mommy.make(models.VatRate, rate=Decimal(10))
        vat_rate2 = mommy.make(models.VatRate, rate=Decimal("5.5"))
        mommy.make(
            models.SaleItem, sale=action1.sale, vat_rate=vat_rate1, pre_tax_price=Decimal(10), quantity=1
        )
        mommy.make(
            models.SaleItem, sale=action1.sale, vat_rate=vat_rate2, pre_tax_price=Decimal(20), quantity=10
        )
        mommy.make(
            models.SaleItem, sale=action2.sale, vat_rate=vat_rate2, pre_tax_price=Decimal(100), quantity=1
        )

        url = reverse('store_actions_summary_xls', args=[action_type.id, '2018-06-01', '2018-06-30'])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        workbook = xlrd.open_workbook(file_contents=response.content, formatting_info=True)
        sheet = workbook.sheet_by_index(0)

        self.assertEqual(sheet.nrows, 3 + 2)

        values = [sheet.cell(1, col).value for col in range(sheet.ncols)]
        self.assertEqual(values[0], 1)
        self.assertEqual(values[1], "Pierre Dupond")
        self.assertEqual(datetime(*xlrd.xldate_as_tuple(values[2], workbook.datemode)).date(), planned_date1.date())
        self.assertEqual(values[3], action_status1.name)
        self.assertEqual(values[4], '')
        self.assertEqual(values[5], Decimal(210))
        self.assertEqual(values[6], Decimal("12"))
        self.assertEqual(values[7], Decimal("222"))

        values = [sheet.cell(2, col).value for col in range(sheet.ncols)]
        self.assertEqual(values[0], 2)
        self.assertEqual(values[1], "Paul Dupuis")
        self.assertEqual(datetime(*xlrd.xldate_as_tuple(values[2], workbook.datemode)).date(), planned_date2.date())
        self.assertEqual(values[3], action_status2.name)
        self.assertEqual(values[4], '')
        self.assertEqual(values[5], Decimal(100))
        self.assertEqual(values[6], Decimal("5.5"))
        self.assertEqual(values[7], Decimal("105.5"))

    def test_export_one_for_entity(self):
        """It should export all in xls"""
        action_type = mommy.make(ActionType, name="Bills of Month", number_auto_generated=1)
        action_status = mommy.make(ActionStatus, name="Created")
        action_status = mommy.make(ActionStatus, name="Paid")

        mommy.make(models.StoreManagementActionType, action_type=action_type)

        entity = mommy.make(Entity, name="Horse&Co")

        planned_date = datetime(2018, 6, 21)
        action = mommy.make(Action, type=action_type, planned_date=planned_date, status=action_status)
        action.entities.add(entity)
        action.save()

        vat_rate = mommy.make(models.VatRate, rate=Decimal(10))
        sale_item = mommy.make(
            models.SaleItem, sale=action.sale, vat_rate=vat_rate, pre_tax_price=Decimal(10), quantity=1
        )

        url = reverse('store_actions_summary_xls', args=[action_type.id, '2018-06-01', '2018-06-30'])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        workbook = xlrd.open_workbook(file_contents=response.content, formatting_info=True)
        sheet = workbook.sheet_by_index(0)

        self.assertEqual(sheet.nrows, 2 + 2)

        values = [sheet.cell(1, col).value for col in range(sheet.ncols)]
        self.assertEqual(values[0], 1)
        self.assertEqual(values[1], "Horse&Co")
        self.assertEqual(datetime(*xlrd.xldate_as_tuple(values[2], workbook.datemode)).date(), planned_date.date())
        self.assertEqual(values[3], action_status.name)
        self.assertEqual(values[4], '')
        self.assertEqual(values[5], Decimal(10))
        self.assertEqual(values[6], Decimal(1))
        self.assertEqual(values[7], Decimal(11))

    def test_export_accounting(self):
        """It should export all in xls"""
        action_type = mommy.make(ActionType, name="Bills of Month", number_auto_generated=1)
        action_status = mommy.make(ActionStatus, name="Created")
        action_status = mommy.make(ActionStatus, name="Paid")

        cat1 = mommy.make(models.StoreItemCategory, accounting_code="code1")
        cat2 = mommy.make(models.StoreItemCategory, accounting_code="code2")
        item1 = mommy.make(models.StoreItem, item_accounting_code="code1")
        item2 = mommy.make(models.StoreItem, item_accounting_code="", category=cat1)
        item3 = mommy.make(models.StoreItem, item_accounting_code="", category=cat2)
        item4 = mommy.make(models.StoreItem, item_accounting_code="code1", category=cat2)

        mommy.make(models.StoreManagementActionType, action_type=action_type)

        entity = mommy.make(Entity, name="Horse&Co")

        planned_date = datetime(2018, 6, 21)
        action = mommy.make(Action, type=action_type, planned_date=planned_date, status=action_status)
        action.entities.add(entity)
        action.save()

        vat_rate1 = mommy.make(models.VatRate, rate=Decimal(10))
        sale_item1 = mommy.make(
            models.SaleItem, sale=action.sale, vat_rate=vat_rate1, pre_tax_price=Decimal(10), quantity=1, item=item1
        )  # code1

        vat_rate2 = mommy.make(models.VatRate, rate=Decimal(20))
        sale_item2 = mommy.make(
            models.SaleItem, sale=action.sale, vat_rate=vat_rate2, pre_tax_price=Decimal(20), quantity=2, item=item2
        )  # code1
        sale_item3 = mommy.make(
            models.SaleItem, sale=action.sale, vat_rate=vat_rate2, pre_tax_price=Decimal(20), quantity=1, item=item3
        )  # code2
        sale_item4 = mommy.make(
            models.SaleItem, sale=action.sale, vat_rate=vat_rate2, pre_tax_price=Decimal(10), quantity=1, item=item4
        )  # code1

        url = reverse('store_actions_summary_xls', args=[action_type.id, '2018-06-01', '2018-06-30'])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        workbook = xlrd.open_workbook(file_contents=response.content, formatting_info=True)
        sheet = workbook.sheet_by_index(0)

        self.assertEqual(sheet.nrows, 3 + 2)

        values = [sheet.cell(1, col).value for col in range(sheet.ncols)]
        self.assertEqual(values[0], 1)
        self.assertEqual(values[1], "Horse&Co")
        self.assertEqual(datetime(*xlrd.xldate_as_tuple(values[2], workbook.datemode)).date(), planned_date.date())
        self.assertEqual(values[3], action_status.name)
        self.assertEqual(values[4], 'code1')
        self.assertEqual(values[5], Decimal(60))
        self.assertEqual(values[6], Decimal(11))
        self.assertEqual(values[7], Decimal(71))

        values = [sheet.cell(2, col).value for col in range(sheet.ncols)]
        self.assertEqual(values[0], 1)
        self.assertEqual(values[1], "Horse&Co")
        self.assertEqual(datetime(*xlrd.xldate_as_tuple(values[2], workbook.datemode)).date(), planned_date.date())
        self.assertEqual(values[3], action_status.name)
        self.assertEqual(values[4], 'code2')
        self.assertEqual(values[5], Decimal(20))
        self.assertEqual(values[6], Decimal(4))
        self.assertEqual(values[7], Decimal(24))





# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.forms.forms import BoundField
from django.utils.translation import ugettext, ugettext_lazy as _

import floppyforms.__future__ as forms

from coop_cms.bs_forms import BootstrapableMixin

from balafon.Crm import models
from balafon.Crm.widgets import CityAutoComplete
from balafon.Crm.utils import get_default_country


class Fieldset(object):

    def __init__(self, name, fields, legend, classes):
        self.name = name
        self.fields = fields
        self.legend = legend
        self.classes = classes

    def __iter__(self):
        for field in self.fields:
            yield field


class FormWithFieldsetMixin(object):

    def get_fieldsets(self):
        """return the list of fieldsets"""
        return self.Meta.fieldsets

    @property
    def fieldsets(self):
        for fieldset_name, fieldset_attrs in self.get_fieldsets():

            fields = []
            for field_name in fieldset_attrs.get('fields', None) or []:
                if field_name in self.fields:
                    fields.append(
                        BoundField(self, self.fields[field_name], field_name)
                    )

            yield Fieldset(
                fieldset_name,
                fields=fields,
                legend=fieldset_attrs.get('legend', ''),
                classes=fieldset_attrs.get('classes', ''),
            )


class BsPopupForm(forms.Form, BootstrapableMixin):
    def __init__(self, *args, **kwargs):
        super(BsPopupForm, self).__init__(*args, **kwargs)
        self._bs_patch_field_class()


class BetterBsForm(forms.Form, BootstrapableMixin):
    """Base class inherit from Bootstrap and form-utils BetterForm"""

    class Media:
        """Media files"""
        css = {
            'all': ('chosen/chosen.css',)
        }
        js = (
            'chosen/chosen.jquery.js',
        )

    def __init__(self, *args, **kwargs):
        super(BetterBsForm, self).__init__(*args, **kwargs)
        self._bs_patch_field_class()
        for field in self.fields.values():
            if field.widget.__class__.__name__ == forms.Select().__class__.__name__:
                field_class = field.widget.attrs.get("class", "")
                if not "chosen-select" in field_class:
                    field.widget.attrs["class"] = field_class + " chosen-select"


class BsPopupModelForm(forms.ModelForm, BootstrapableMixin):  # (BetterModelForm):
    """Base class inherit from Bootstrap and form-utils BetterModelForm"""

    def __init__(self, *args, **kwargs):
        super(BsPopupModelForm, self).__init__(*args, **kwargs)
        self._bs_patch_field_class()


class BetterBsModelForm(forms.ModelForm, BootstrapableMixin):  # (BetterModelForm):
    """Base class inherit from Bootstrap and form-utils BetterModelForm"""

    class Media:
        """Media files"""
        css = {
            'all': ('chosen/chosen.css',)
        }
        js = (
            'chosen/chosen.jquery.js',
        )

    def __init__(self, *args, **kwargs):
        super(BetterBsModelForm, self).__init__(*args, **kwargs)
        self._bs_patch_field_class()
        for field in self.fields.values():
            if field.widget.__class__.__name__ == forms.Select().__class__.__name__:
                css_class = field.widget.attrs.get("class", "")
                if "chosen-select" not in css_class:
                    field.widget.attrs["class"] = css_class + " chosen-select"


class _CityBasedForm(object):
    """Base class for form with a City field"""

    def __init__(self, *args, **kwargs):
        self.country_id = 0

    def _get_city_parent(self, city):
        """get city parent"""
        parent = city.parent
        country = None
        while parent:
            country = parent
            parent = parent.parent
        return country

    def _manage_country_field(self, field_prefix, *args, **kwargs):
        """"""
        setattr(self, field_prefix + 'country_id', 0)
        if len(args):
            try:
                setattr(self, field_prefix + 'country_id', int(args[0][field_prefix + "country"]))
            except (KeyError, ValueError):
                pass
        if not getattr(self, field_prefix + 'country_id', 0):
            setattr(self, field_prefix + 'country_id', get_default_country().id)

        self.fields[field_prefix + 'city'].widget = CityAutoComplete(
            attrs={'placeholder': _('Enter a city'), 'size': '80'}
        )

        zones_choices = [
            (zone.id, zone.name) for zone in models.Zone.objects.filter(parent__isnull=True).order_by('name')
        ]
        self.fields[field_prefix + 'country'].choices = [(0, '')] + zones_choices

        try:
            city = getattr(kwargs.get('instance'), field_prefix + 'city', 0)
            if city:
                city_parent = self._get_city_parent(city)
                if city_parent:
                    self.fields[field_prefix + 'country'].initial = city_parent.id
        except models.City.DoesNotExist:
            pass

    def _post_init(self, *args, **kwargs):
        """must be called at the end of __init__ of the subclasses"""
        self._manage_country_field('', *args, **kwargs)

    def _get_country(self, country_id):
        """get the country"""
        if country_id:
            return models.Zone.objects.get(id=country_id, parent__isnull=True, type__type="country")
        else:
            return get_default_country()

    def _clean_city_field(self, field_prefix):
        """city validation"""
        city = self.cleaned_data[field_prefix + 'city']
        if isinstance(city, models.City):
            return city
        else:
            try:
                if not city:
                    return None
                try:
                    city_id = int(city)
                    return models.City.objects.get(id=city_id)
                except (ValueError, TypeError):
                    pass

                zip_code = self.cleaned_data[field_prefix + 'zip_code']

                try:
                    country_id = int(self.cleaned_data.get(field_prefix + 'country')) \
                        or getattr(self, field_prefix + 'country_id')
                except (ValueError, TypeError):
                    country_id = getattr(self, field_prefix + 'country_id')

                country = self._get_country(country_id)
                default_country = get_default_country()

                if country != default_country:
                    city = models.City.objects.get_or_create(name=city, parent=country)[0]
                else:
                    if len(zip_code) < 2:
                        raise ValidationError(ugettext('You must enter a valid zip code for selecting a new city'))
                    dep = models.Zone.objects.get(code=zip_code[:2])
                    city = models.City.objects.get_or_create(name=city, parent=dep)[0]
                return city
            except ValidationError:
                raise
            except Exception as msg:
                raise ValidationError(msg)

    def clean_city(self):
        """city validation"""
        return self._clean_city_field('')


class ModelFormWithCity(BetterBsModelForm, _CityBasedForm):
    """ModelForm with city"""
    city = forms.CharField(
        required=False,
        label=_('City'),
        widget=CityAutoComplete(attrs={'placeholder': _('Enter a city'), 'size': '80'})
    )
    country = forms.ChoiceField(required=False, label=_('Country'))

    def __init__(self, *args, **kwargs):
        super(ModelFormWithCity, self).__init__(*args, **kwargs)
        self._post_init(*args, **kwargs)


class FormWithCity(BetterBsForm, _CityBasedForm):
    """Form with city"""

    country = forms.ChoiceField(required=False, label=_('Country'))
    zip_code = forms.CharField(required=False, label=_('zip code'))
    city = forms.CharField(required=False, label=_('City'))

    def __init__(self, *args, **kwargs):
        super(FormWithCity, self).__init__(*args, **kwargs)
        self._post_init(*args, **kwargs)


class BillingModelFormWithCity(ModelFormWithCity):
    """ModelForm with city"""
    billing_city = forms.CharField(
        required=False,
        label=_('City'),
        widget=CityAutoComplete(attrs={'placeholder': _('Enter a city'), 'size': '80'})
    )
    billing_country = forms.ChoiceField(required=False, label=_('Country'))

    def __init__(self, *args, **kwargs):
        super(BillingModelFormWithCity, self).__init__(*args, **kwargs)

    def _post_init(self, *args, **kwargs):
        """called at the end of __init__ of the parent class"""
        super(BillingModelFormWithCity, self)._post_init(*args, **kwargs)
        self._manage_country_field('billing_', *args, **kwargs)

    def clean_billing_city(self):
        """city validation"""
        return self._clean_city_field('billing_')


class ModelFormWithAddress(BillingModelFormWithCity):
    """ModelForm with address"""

    def __init__(self, *args, **kwargs):
        instance = kwargs.get('instance', None)
        if instance:
            prefix = " ".join([
                instance.street_number,
                instance.street_type.name if instance.street_type else '',
            ]).strip()
            instance.address = instance.address.replace(prefix, '', 1).strip()

            billing_prefix = " ".join([
                instance.billing_street_number,
                instance.billing_street_type.name if instance.billing_street_type else '',
            ]).strip()
            instance.billing_address = instance.billing_address.replace(billing_prefix, '', 1).strip()

        super(ModelFormWithAddress, self).__init__(*args, **kwargs)

        if not models.StreetType.objects.count():
            self.fields['street_number'].widget = forms.HiddenInput()
            self.fields['street_type'].widget = forms.HiddenInput()
            self.fields['billing_street_number'].widget = forms.HiddenInput()
            self.fields['billing_street_type'].widget = forms.HiddenInput()

    def clean_address(self):
        """add street type and number to the address"""
        street_number = self.cleaned_data['street_number']
        street_type = self.cleaned_data['street_type']
        address = self.cleaned_data['address']
        if street_number or street_type:
            return " ".join([street_number, street_type.name if street_type else '', address]).strip()
        return address

    def clean_billing_address(self):
        """add street type and number to the address"""
        billing_street_number = self.cleaned_data['billing_street_number']
        billing_street_type = self.cleaned_data['billing_street_type']
        billing_address = self.cleaned_data['billing_address']
        if billing_street_number or billing_street_type:
            return " ".join(
                [billing_street_number, billing_street_type.name if billing_street_type else '', billing_address]
            ).strip()
        return billing_address



# -*- coding: utf-8 -*-
"""unit testing"""

from __future__ import unicode_literals

from datetime import datetime, timedelta

from django.core.urlresolvers import reverse

from coop_cms.tests import BeautifulSoup
from model_mommy import mommy

from balafon.unit_tests import response_as_json
from balafon.Crm import models
from balafon.Crm.tests import BaseTestCase


class ActionArchiveTest(BaseTestCase):
    """test view planning"""

    def test_view_monthly_action_anonymous(self):
        """make sure not display for anonymous users"""
        mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now())
        self.client.logout()

        now = datetime.now()
        url = reverse('crm_actions_of_month', args=[now.year, now.month])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 302)
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

    def test_view_monthly_action_non_staff(self):
        """make sure not display for anonymous users"""
        mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now())
        self.user.is_staff = False
        self.user.save()

        now = datetime.now()
        url = reverse('crm_actions_of_month', args=[now.year, now.month])
        response = self.client.get(url)
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

    def test_view_monthly_action(self):
        """view actions for the month"""
        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now())
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=datetime.now())
        action3 = mommy.make(models.Action, subject="#ACT3#", planned_date=datetime.now()+timedelta(days=31))
        action4 = mommy.make(models.Action, subject="#ACT4#", planned_date=datetime.now()-timedelta(days=31))
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=None)

        date_now = datetime.now()
        url = reverse('crm_actions_of_month', args=[date_now.year, date_now.month])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertContains(response, action2.subject)
        self.assertNotContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)

    def test_view_monthly_action_end_dt(self):
        """view actions ending in the month"""
        action1 = mommy.make(
            models.Action, subject="#ACT1#", planned_date=datetime(2014, 4, 29), end_datetime=datetime(2014, 5, 2)
        )
        action2 = mommy.make(
            models.Action, subject="#ACT2#", planned_date=datetime(2014, 3, 29), end_datetime=datetime(2014, 4, 2)
        )
        action3 = mommy.make(
            models.Action, subject="#ACT3#", planned_date=datetime(2014, 3, 29), end_datetime=datetime(2014, 3, 30)
        )
        action4 = mommy.make(
            models.Action, subject="#ACT4#", planned_date=datetime(2014, 5, 1), end_datetime=datetime(2014, 5, 2)
        )
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=None)
        action6 = mommy.make(
            models.Action, subject="#ACT6#", planned_date=datetime(2014, 4, 2), end_datetime=datetime(2014, 4, 8)
        )
        action7 = mommy.make(
            models.Action, subject="#ACT7#", planned_date=datetime(2014, 3, 29), end_datetime=datetime(2014, 5, 2)
        )
        action8 = mommy.make(models.Action, subject="#ACT8#", planned_date=datetime(2014, 4, 29))

        url = reverse('crm_actions_of_month', args=[2014, 4])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertContains(response, action2.subject)
        self.assertNotContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)
        self.assertContains(response, action6.subject)
        self.assertContains(response, action7.subject)
        self.assertContains(response, action8.subject)

    def test_view_weekly_action_anonymous(self):
        """make sure not display for anonymous users"""
        mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now())
        self.client.logout()

        now = datetime.now()
        url = reverse('crm_actions_of_week', args=[now.year, now.strftime("%W")])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 302)
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

    def test_view_weekly_action_non_staff(self):
        """make sure not display for anonymous users"""
        mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now())
        self.user.is_staff = False
        self.user.save()

        now = datetime.now()
        url = reverse('crm_actions_of_week', args=[now.year, now.strftime("%W")])
        response = self.client.get(url)
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

    def test_view_weekly_action(self):
        """view actions of the week"""

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now())
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=datetime.now())
        action3 = mommy.make(models.Action, subject="#ACT3#", planned_date=datetime.now()+timedelta(days=7))
        action4 = mommy.make(models.Action, subject="#ACT4#", planned_date=datetime.now()-timedelta(days=7))
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=None)

        now = datetime.now()
        url = reverse('crm_actions_of_week', args=[now.year, now.strftime("%W")])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertContains(response, action2.subject)
        self.assertNotContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)

    def test_view_weekly_action_end_dt(self):
        """view actions ending in the week"""
        action1 = mommy.make(
            models.Action, subject="#ACT1#", planned_date=datetime(2014, 4, 9), end_datetime=datetime(2014, 4, 15)
        )
        action2 = mommy.make(
            models.Action, subject="#ACT2#", planned_date=datetime(2014, 4, 1), end_datetime=datetime(2014, 4, 9)
        )
        action3 = mommy.make(
            models.Action, subject="#ACT3#", planned_date=datetime(2014, 4, 1), end_datetime=datetime(2014, 4, 2)
        )
        action4 = mommy.make(
            models.Action, subject="#ACT4#", planned_date=datetime(2014, 4, 15), end_datetime=datetime(2014, 4, 16)
        )
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=None)
        action6 = mommy.make(
            models.Action, subject="#ACT6#", planned_date=datetime(2014, 4, 2), end_datetime=datetime(2014, 4, 8)
        )
        action7 = mommy.make(
            models.Action, subject="#ACT7#", planned_date=datetime(2014, 3, 29), end_datetime=datetime(2014, 5, 2)
        )
        action8 = mommy.make(models.Action, subject="#ACT8#", planned_date=datetime(2014, 4, 9))

        url = reverse('crm_actions_of_week', args=[2014, 14])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertContains(response, action2.subject)
        self.assertNotContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)
        self.assertContains(response, action6.subject)
        self.assertContains(response, action7.subject)
        self.assertContains(response, action8.subject)

    def test_view_daily_action_anonymous(self):
        """make sure not display for anonymous users"""
        mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now())
        self.client.logout()

        now = datetime.now()
        url = reverse('crm_actions_of_day', args=[now.year, now.month, now.day])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 302)
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

    def test_view_dailly_action_non_staff(self):
        """make sure not display for anonymous users"""
        mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now())
        self.user.is_staff = False
        self.user.save()

        now = datetime.now()
        url = reverse('crm_actions_of_day', args=[now.year, now.month, now.day])
        response = self.client.get(url)
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

    def test_view_daily_action(self):
        """view actions of the day"""

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now())
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=datetime.now())
        action3 = mommy.make(models.Action, subject="#ACT3#", planned_date=datetime.now()+timedelta(days=1))
        action4 = mommy.make(models.Action, subject="#ACT4#", planned_date=datetime.now()-timedelta(days=1))
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=None)

        now = datetime.now()
        url = reverse('crm_actions_of_day', args=[now.year, now.month, now.day])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertContains(response, action2.subject)
        self.assertNotContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)

    def test_view_daily_action_end_dt(self):
        """view actions ending in the week"""

        action1 = mommy.make(
            models.Action, subject="#ACT1#", planned_date=datetime(2014, 4, 9), end_datetime=datetime(2014, 4, 9)
        )
        action2 = mommy.make(
            models.Action, subject="#ACT2#", planned_date=datetime(2014, 4, 8), end_datetime=datetime(2014, 4, 12)
        )
        action3 = mommy.make(
            models.Action, subject="#ACT3#", planned_date=datetime(2014, 4, 8), end_datetime=datetime(2014, 4, 8)
        )
        action4 = mommy.make(
            models.Action, subject="#ACT4#", planned_date=datetime(2014, 4, 10), end_datetime=datetime(2014, 4, 10)
        )
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=None)
        action6 = mommy.make(
            models.Action, subject="#ACT6#", planned_date=datetime(2014, 4, 2), end_datetime=datetime(2014, 4, 9)
        )
        action7 = mommy.make(
            models.Action, subject="#ACT7#", planned_date=datetime(2014, 4, 9), end_datetime=datetime(2014, 4, 10)
        )
        action8 = mommy.make(models.Action, subject="#ACT8#", planned_date=datetime(2014, 4, 9))

        url = reverse('crm_actions_of_day', args=[2014, 4, 9])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertContains(response, action2.subject)
        self.assertNotContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)
        self.assertContains(response, action6.subject)
        self.assertContains(response, action7.subject)
        self.assertContains(response, action8.subject)

    def test_in_charge_filter(self):
        """view actions of the month in charge filter"""

        user_joe = mommy.make(models.TeamMember, name="Joe")
        user_jack = mommy.make(models.TeamMember, name="Jack")
        user_will = mommy.make(models.TeamMember, name="Will")

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now(), in_charge=user_joe)
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=datetime.now(), in_charge=user_joe)
        action3 = mommy.make(models.Action, subject="#ACT3#", planned_date=datetime.now(), in_charge=user_jack)
        action4 = mommy.make(models.Action, subject="#ACT4#", planned_date=datetime.now(), in_charge=user_will)
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=datetime.now())

        date_now = datetime.now()
        url = reverse(
            'crm_actions_of_month', args=[date_now.year, date_now.month]
        ) + "?filter=u{0},u{1}".format(user_joe.id, user_jack.id)
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertContains(response, action2.subject)
        self.assertContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(
            ["u{0}".format(y.id) for y in [user_joe, user_jack]],
            [x["value"] for x in soup.select("select.action-filter option[selected=selected]")]
        )

    def test_type_filter(self):
        """view actions of the month action type filter"""

        action_type1 = mommy.make(models.ActionType, name='Abc')
        action_type2 = mommy.make(models.ActionType, name='Abd')
        action_type3 = mommy.make(models.ActionType, name='Abe')

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now(), type=action_type1)
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=datetime.now(), type=action_type1)
        action3 = mommy.make(models.Action, subject="#ACT3#", planned_date=datetime.now(), type=action_type2)
        action4 = mommy.make(models.Action, subject="#ACT4#", planned_date=datetime.now(), type=action_type3)
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=datetime.now())

        now = datetime.now()
        url = reverse(
            'crm_actions_of_month', args=[now.year, now.month]
        ) + "?filter=t{0},t{1}".format(action_type1.id, action_type2.id)
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertContains(response, action2.subject)
        self.assertContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(
            sorted(["t{0}".format(y.id) for y in [action_type1, action_type2]]),
            sorted([x["value"] for x in soup.select("select.action-filter option[selected=selected]")])
        )

    def test_action_type_none(self):
        """view actions of the month no type"""

        action_type1 = mommy.make(models.ActionType, name='Abc')
        action_type2 = mommy.make(models.ActionType, name='Abd')
        action_type3 = mommy.make(models.ActionType, name='Abe')

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now(), type=action_type1)
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=datetime.now(), type=action_type1)
        action3 = mommy.make(models.Action, subject="#ACT3#", planned_date=datetime.now(), type=action_type2)
        action4 = mommy.make(models.Action, subject="#ACT4#", planned_date=datetime.now(), type=action_type3)
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=datetime.now())

        now = datetime.now()
        url = reverse('crm_actions_of_month', args=[now.year, now.month]) + "?filter=t0"
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, action1.subject)
        self.assertNotContains(response, action2.subject)
        self.assertNotContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertContains(response, action5.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(["t0"], [x["value"] for x in soup.select("select.action-filter option[selected=selected]")])

    def test_type_none_and_defined(self):
        """view actions of the month action type some are set"""

        action_type1 = mommy.make(models.ActionType, name='Abc')
        action_type2 = mommy.make(models.ActionType, name='Abd')
        action_type3 = mommy.make(models.ActionType, name='Abe')

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now(), type=action_type1)
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=datetime.now(), type=action_type1)
        action3 = mommy.make(models.Action, subject="#ACT3#", planned_date=datetime.now(), type=action_type2)
        action4 = mommy.make(models.Action, subject="#ACT4#", planned_date=datetime.now(), type=action_type3)
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=datetime.now())

        now = datetime.now()
        url = reverse('crm_actions_of_month', args=[now.year, now.month]) + "?filter=t0,t{0}".format(action_type1.id)
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertContains(response, action2.subject)
        self.assertNotContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertContains(response, action5.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(
            ["t0"] + ["t{0}".format(y.id) for y in [action_type1,]],
            [x["value"] for x in soup.select("select.action-filter option[selected=selected]")]
        )

    def test_filter_invalid(self):
        """view actions of the month : invalid filter"""

        mommy.make(models.Action, subject="#ACT5#", planned_date=datetime.now())

        now = datetime.now()
        url = reverse('crm_actions_of_month', args=[now.year, now.month]) + "?filter=abc"
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 404)

    def test_filter_invalid_user(self):
        """view actions of the month filter with invalid user"""

        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=datetime.now())

        now = datetime.now()
        url = reverse('crm_actions_of_month', args=[now.year, now.month]) + "?filter=u2"
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, action5.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(
            [],
            [x["value"] for x in soup.select("select.action-filter option[selected=selected]")]
        )

    def test_filter_invalid_type(self):
        """view actions of the month filter with invalid type"""
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=datetime.now())

        now = datetime.now()
        url = reverse('crm_actions_of_month', args=[now.year, now.month]) + "?filter=t2"
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, action5.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(
            [],
            [x["value"] for x in soup.select("select.action-filter option[selected=selected]")]
        )

    def test_action_type_in_charge_filter(self):
        """view actions of the month incharge filter"""
        user_joe = mommy.make(models.TeamMember, name="Joe")
        action_type1 = mommy.make(models.ActionType, name='Abc')
        action_type2 = mommy.make(models.ActionType, name='Abd')

        action1 = mommy.make(
            models.Action, subject="#ACT1#", planned_date=datetime.now(), in_charge=user_joe, type=action_type1
        )
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=datetime.now(), in_charge=user_joe)
        action3 = mommy.make(models.Action, subject="#ACT3#", planned_date=datetime.now(), type=action_type1)
        action4 = mommy.make(models.Action, subject="#ACT4#", planned_date=datetime.now(), type=action_type2)
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=datetime.now())

        now = datetime.now()
        url = reverse(
            'crm_actions_of_month', args=[now.year, now.month]
        ) + "?filter=u{0},t{1}".format(user_joe.id, action_type1.id)
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertNotContains(response, action2.subject)
        self.assertNotContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(
            ["t{0}".format(action_type1.id)] + ["u{0}".format(user_joe.id)],
            [x["value"] for x in soup.select("select.action-filter option[selected=selected]")]
        )

    def test_action_type_and_status(self):
        """view actions of the month incharge filter"""
        action_type1 = mommy.make(models.ActionType, name='XYZ')
        action_type2 = mommy.make(models.ActionType, name="UVW")
        action_status1 = mommy.make(models.ActionStatus, name='Def')
        action_status2 = mommy.make(models.ActionStatus, name='Abc')
        action_type1.allowed_status.add(action_status1)
        action_type1.allowed_status.add(action_status2)
        action_type1.save()
        action_type2.allowed_status.add(action_status2)
        action_type2.save()

        action1 = mommy.make(
            models.Action, subject="#ACT1#", planned_date=datetime.now(), status=action_status1, type=action_type1
        )
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=datetime.now(), type=action_type1)
        action3 = mommy.make(
            models.Action, subject="#ACT3#", planned_date=datetime.now(), status=action_status2, type=action_type1
        )
        action4 = mommy.make(
            models.Action, subject="#ACT4#", planned_date=datetime.now(), status=action_status2, type=action_type2
        )
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=datetime.now())

        now = datetime.now()
        url = reverse(
            'crm_actions_of_month', args=[now.year, now.month]
        ) + "?filter=s{0},t{1}".format(action_status2.id, action_type1.id)
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, action1.subject)
        self.assertNotContains(response, action2.subject)
        self.assertContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(
            ["t{0}".format(action_type1.id)] + ["s{0}".format(action_status2.id)],
            [x["value"] for x in soup.select("select.action-filter option[selected=selected]")]
        )
        self.assertEqual(
            [
                "t0",
                "t{0}".format(action_type2.id), "t{0}".format(action_type1.id),
                "s{0}".format(action_status2.id), "s{0}".format(action_status1.id)
            ],
            [x["value"] for x in soup.select("select.action-filter option")]
        )

    def test_action_type_and_not_managed_status(self):
        """view actions of the month incharge filter"""
        action_type1 = mommy.make(models.ActionType, name='Abc')
        action_type2 = mommy.make(models.ActionType, name='Abd')
        action_status1 = mommy.make(models.ActionStatus)
        action_status2 = mommy.make(models.ActionStatus)

        action1 = mommy.make(
            models.Action, subject="#ACT1#", planned_date=datetime.now(), status=action_status1, type=action_type1
        )
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=datetime.now(), type=action_type1)
        action3 = mommy.make(
            models.Action, subject="#ACT3#", planned_date=datetime.now(), status=action_status2, type=action_type1
        )
        action4 = mommy.make(
            models.Action, subject="#ACT4#", planned_date=datetime.now(), status=action_status2, type=action_type2
        )
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=datetime.now())

        now = datetime.now()
        url = reverse(
            'crm_actions_of_month', args=[now.year, now.month]
        ) + "?filter=s{0},t{1}".format(action_status2.id, action_type1.id)
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertContains(response, action2.subject)
        self.assertContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(
            ["t{0}".format(action_type1.id)],
            [x["value"] for x in soup.select("select.action-filter option[selected=selected]")]
        )
        self.assertEqual(
            ["t0", "t{0}".format(action_type1.id), "t{0}".format(action_type2.id)],
            [x["value"] for x in soup.select("select.action-filter option")]
        )

    def test_action_type_ordering_asc(self):
        """view actions of the month incharge filter"""
        now = datetime(2016, 10, 10, 12, 0)

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=now + timedelta(days=1))
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=now + timedelta(days=2))
        action3 = mommy.make(models.Action, subject="#ACT3#", planned_date=now - timedelta(days=1))
        action4 = mommy.make(models.Action, subject="#ACT4#", planned_date=now - timedelta(days=3))
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=now)

        url = reverse(
            'crm_actions_of_month', args=[now.year, now.month]
        ) + "?filter=o1"
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)

        actions = [
            action4, action3, action5, action1, action2
        ]

        subjects = [action.subject for action in actions]
        for subject in subjects:
            self.assertContains(response, subject)
        content = response.content.decode('utf-8')
        pos = [content.find(subject) for subject in subjects]
        self.assertEqual(pos, list(sorted(pos)))

    def test_action_type_ordering_desc(self):
        """view actions of the month incharge filter"""
        now = datetime(2016, 10, 10, 12, 0)

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=now + timedelta(days=1))
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=now + timedelta(days=2))
        action3 = mommy.make(models.Action, subject="#ACT3#", planned_date=now - timedelta(days=1))
        action4 = mommy.make(models.Action, subject="#ACT4#", planned_date=now - timedelta(days=3))
        action5 = mommy.make(models.Action, subject="#ACT5#", planned_date=now)

        url = reverse(
            'crm_actions_of_month', args=[now.year, now.month]
        ) + "?filter=o0"
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)

        actions = [
            action2, action1, action5, action3, action4
        ]

        subjects = [action.subject for action in actions]
        for subject in subjects:
            self.assertContains(response, subject)
        content = response.content.decode('utf-8')
        pos = [content.find(subject) for subject in subjects]
        self.assertEqual(pos, list(sorted(pos)))

    def test_view_not_planned_action_anonymous(self):
        """make sure not display for anonymous users"""
        mommy.make(models.Action, subject="#ACT1#")
        self.client.logout()

        url = reverse('crm_actions_not_planned')
        response = self.client.get(url)

        self.assertEqual(response.status_code, 302)
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

    def test_view_not_planned_action_non_staff(self):
        """make sure not display for anonymous users"""
        mommy.make(models.Action, subject="#ACT1#")
        self.user.is_staff = False
        self.user.save()

        url = reverse('crm_actions_not_planned')
        response = self.client.get(url)
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

    def test_view_not_planned_action(self):
        """view actions not planned"""
        user_joe = mommy.make(models.TeamMember, name="Joe")
        action_type1 = mommy.make(models.ActionType, name='Abc')
        action_type2 = mommy.make(models.ActionType, name='Abd')

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=None, in_charge=user_joe, type=action_type1)
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=None, in_charge=user_joe)
        action3 = mommy.make(models.Action, subject="#ACT3#", planned_date=None, type=action_type1)
        action4 = mommy.make(models.Action, subject="#ACT4#", planned_date=None, type=action_type2)
        action5 = mommy.make(
            models.Action, subject="#ACT5#", planned_date=datetime.now(), in_charge=user_joe, type=action_type1
        )

        url = reverse('crm_actions_not_planned')
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertContains(response, action2.subject)
        self.assertContains(response, action3.subject)
        self.assertContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(
            [],
            [x["value"] for x in soup.select("select.action-filter option[selected=selected]")]
        )

    def test_not_planned_type_in_charge_filter(self):
        """view actions not planned filtered by in charge"""
        user_joe = mommy.make(models.TeamMember, name="Joe")
        action_type1 = mommy.make(models.ActionType, name='Abc')
        action_type2 = mommy.make(models.ActionType, name='Abd')

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=None, in_charge=user_joe, type=action_type1)
        action2 = mommy.make(models.Action, subject="#ACT2#", planned_date=None, in_charge=user_joe)
        action3 = mommy.make(models.Action, subject="#ACT3#", planned_date=None, type=action_type1)
        action4 = mommy.make(models.Action, subject="#ACT4#", planned_date=None, type=action_type2)
        action5 = mommy.make(
            models.Action, subject="#ACT5#", planned_date=datetime.now(), in_charge=user_joe, type=action_type1
        )

        url = reverse('crm_actions_not_planned') + "?filter=u{0},t{1}".format(user_joe.id, action_type1.id)
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)
        self.assertNotContains(response, action2.subject)
        self.assertNotContains(response, action3.subject)
        self.assertNotContains(response, action4.subject)
        self.assertNotContains(response, action5.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(
            ["t{0}".format(action_type1.id)] + ["u{0}".format(user_joe.id)],
            [x["value"] for x in soup.select("select.action-filter option[selected=selected]")]
        )

    def test_view_action_final_status(self):
        """It should show change status when clicking on done button"""

        status1 = mommy.make(models.ActionStatus, is_final=True)
        status2 = mommy.make(models.ActionStatus, is_final=False)

        action_type = mommy.make(models.ActionType)
        action_type.allowed_status.add(status1)
        action_type.allowed_status.add(status2)
        action_type.save()

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now(), type=action_type)

        now = datetime.now()
        url = reverse('crm_actions_of_week', args=[now.year, now.strftime("%W")])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select(".action")), 1)
        self.assertContains(response, reverse('crm_update_action_status', args=[action1.id])+"?done=1")
        self.assertNotContains(response, reverse('crm_do_action', args=[action1.id]))

    def test_view_action_no_final_status(self):
        """It should show change done flag when clicking on done button"""

        status1 = mommy.make(models.ActionStatus, is_final=False)
        status2 = mommy.make(models.ActionStatus, is_final=False)

        action_type = mommy.make(models.ActionType)
        action_type.allowed_status.add(status1)
        action_type.allowed_status.add(status2)
        action_type.save()

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now(), type=action_type)

        now = datetime.now()
        url = reverse('crm_actions_of_week', args=[now.year, now.strftime("%W")])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select(".action")), 1)
        self.assertNotContains(response, reverse('crm_update_action_status', args=[action1.id])+"?done=1")
        self.assertContains(response, reverse('crm_do_action', args=[action1.id]))

    def test_view_action_status_style(self):
        """It should show status color correctly"""

        status1 = mommy.make(models.ActionStatus, fore_color="#fff;", background_color="#000")

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now(), status=status1)

        now = datetime.now()
        url = reverse('crm_actions_of_week', args=[now.year, now.strftime("%W")])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select(".action")), 1)
        self.assertContains(response, "background: #000;")
        self.assertContains(response, "color: #fff;")

    def test_show_action_contacts_buttons(self):
        """It should show add/remove contacts/entities buttons"""

        action_type = mommy.make(models.ActionType)

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now(), type=action_type)

        contact = mommy.make(models.Contact)
        entity = mommy.make(models.Entity)

        action1.contacts.add(contact)
        action1.entities.add(entity)
        action1.save()

        now = datetime.now()
        url = reverse('crm_actions_of_week', args=[now.year, now.strftime("%W")])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, action1.subject)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select(".action")), 1)

        urls = (
            reverse('crm_add_contact_to_action', args=[action1.id]),
            reverse('crm_add_entity_to_action', args=[action1.id]),
            reverse('crm_remove_contact_from_action', args=[action1.id, contact.id]),
            reverse('crm_remove_entity_from_action', args=[action1.id, entity.id]),
        )

        for link_url in urls:
            self.assertContains(response, link_url)

    def test_hide_action_contacts_buttons(self):
        """It should hide add/remove contacts/entities buttons"""

        action_type = mommy.make(models.ActionType, hide_contacts_buttons=True)

        action1 = mommy.make(models.Action, subject="#ACT1#", planned_date=datetime.now(), type=action_type)

        contact = mommy.make(models.Contact)
        entity = mommy.make(models.Entity)

        action1.contacts.add(contact)
        action1.entities.add(entity)
        action1.save()

        now = datetime.now()
        url = reverse('crm_actions_of_week', args=[now.year, now.strftime("%W")])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select(".action")), 1)

        self.assertContains(response, action1.subject)

        urls = (
            reverse('crm_add_contact_to_action', args=[action1.id]),
            reverse('crm_add_entity_to_action', args=[action1.id]),
            reverse('crm_remove_contact_from_action', args=[action1.id, contact.id]),
            reverse('crm_remove_entity_from_action', args=[action1.id, entity.id]),
        )

        for link_url in urls:
            self.assertNotContains(response, link_url)


class PlanningRedirectView(BaseTestCase):
    """Test redirections"""

    def test_view_this_month_actions(self):
        """It should redirect and keep query string args"""
        action_type = mommy.make(models.ActionType)
        query_string = "?t={0}".format(action_type.id)
        url = reverse('crm_this_month_actions') + query_string
        response = self.client.get(url)
        self.assertEqual(302, response.status_code)
        now = datetime.now()
        redirect_url = reverse('crm_actions_of_month', args=[now.year, now.month]) + query_string
        self.assertTrue(response['Location'].find(redirect_url) >= 0)

    def test_view_this_week_actions(self):
        """It should redirect and keep query string args"""
        action_type = mommy.make(models.ActionType)
        query_string = "?t={0}".format(action_type.id)
        url = reverse('crm_this_week_actions') + query_string
        response = self.client.get(url)
        self.assertEqual(302, response.status_code)
        now = datetime.now()
        redirect_url = reverse('crm_actions_of_week', args=[now.year, now.strftime("%W")]) + query_string
        self.assertTrue(response['Location'].find(redirect_url) >= 0)

    def test_view_today_actions(self):
        """It should redirect and keep query string args"""
        action_type = mommy.make(models.ActionType)
        query_string = "?t={0}".format(action_type.id)
        url = reverse('crm_today_actions') + query_string
        response = self.client.get(url)
        self.assertEqual(302, response.status_code)
        now = datetime.now()
        redirect_url = reverse('crm_actions_of_day', args=[now.year, now.month, now.day]) + query_string
        self.assertTrue(response['Location'].find(redirect_url) >= 0)


class GoToPlanningDateTest(BaseTestCase):
    """It should return the url of given parameters"""

    def test_404_on_get(self):
        """It should return 404"""
        url = reverse('crm_go_to_planning_date')
        response = self.client.get(url)
        self.assertEqual(404, response.status_code)

    def test_go_to_day(self):
        """It should return url"""
        url = reverse('crm_go_to_planning_date')

        data = {
            'planning_type': 'day',
            'planning_date': '2016-02-14',
            'filters': 't2',
        }

        response = self.client.post(url, data)
        self.assertEqual(200, response.status_code)

        resp_data = response_as_json(response)

        self.assertEqual(resp_data['url'], reverse('crm_actions_of_day', args=[2016, 2, 14]) + '?filter=t2')

    def test_go_to_day_september(self):
        """It should return url"""
        url = reverse('crm_go_to_planning_date')

        data = {
            'planning_type': 'day',
            'planning_date': '2016-09-14',
            'filters': 't2',
        }

        response = self.client.post(url, data)
        self.assertEqual(200, response.status_code)

        resp_data = response_as_json(response)

        self.assertEqual(resp_data['url'], reverse('crm_actions_of_day', args=[2016, 9, 14]) + '?filter=t2')

    def test_go_to_day_december(self):
        """It should return url"""
        url = reverse('crm_go_to_planning_date')

        data = {
            'planning_type': 'day',
            'planning_date': '2016-12-09',
            'filters': 't2',
        }

        response = self.client.post(url, data)
        self.assertEqual(200, response.status_code)

        resp_data = response_as_json(response)

        self.assertEqual(resp_data['url'], reverse('crm_actions_of_day', args=[2016, 12, 9]) + '?filter=t2')

    def test_go_to_month(self):
        """It should return url"""
        url = reverse('crm_go_to_planning_date')

        data = {
            'planning_type': 'month',
            'planning_date': '2016-02-14',
            'filters': 't2',
        }

        response = self.client.post(url, data)
        self.assertEqual(200, response.status_code)

        resp_data = response_as_json(response)

        self.assertEqual(resp_data['url'], reverse('crm_actions_of_month', args=[2016, 2]) + '?filter=t2')

    def test_go_to_week(self):
        """It should return url"""
        url = reverse('crm_go_to_planning_date')

        data = {
            'planning_type': 'week',
            'planning_date': '2016-02-14',
            'filters': 't2',
        }

        response = self.client.post(url, data)
        self.assertEqual(200, response.status_code)

        resp_data = response_as_json(response)

        self.assertEqual(resp_data['url'], reverse('crm_actions_of_week', args=[2016, 6]) + '?filter=t2')

    def test_go_to_week_no_filters(self):
        """It should return url"""
        url = reverse('crm_go_to_planning_date')

        data = {
            'planning_type': 'week',
            'planning_date': '2016-02-14',
            'filters': '',
        }

        response = self.client.post(url, data)
        self.assertEqual(200, response.status_code)

        resp_data = response_as_json(response)

        self.assertEqual(resp_data['url'], reverse('crm_actions_of_week', args=[2016, 6]))

    def test_go_to_invalid_type(self):
        """It should return 404 error"""
        url = reverse('crm_go_to_planning_date')

        data = {
            'planning_type': 'blabla',
            'planning_date': '2016-02-14',
            'filters': 't2',
        }

        response = self.client.post(url, data)
        self.assertEqual(404, response.status_code)

    def test_go_to_invalid_date(self):
        """It should return 404 error"""
        url = reverse('crm_go_to_planning_date')

        data = {
            'planning_type': 'month',
            'planning_date': 'blabla',
            'filters': 't2',
        }

        response = self.client.post(url, data)
        self.assertEqual(404, response.status_code)

    def test_go_to_anonymous(self):
        """It should not be allowed"""
        url = reverse('crm_go_to_planning_date')
        self.client.logout()
        data = {
            'planning_type': 'month',
            'planning_date': '2016-02-14',
            'filters': 't2',
        }

        response = self.client.post(url, data)
        self.assertEqual(302, response.status_code)

    def test_go_to_not_allowed(self):
        """It should not be allowed"""
        url = reverse('crm_go_to_planning_date')
        self.user.is_staff = False
        self.user.save()

        data = {
            'planning_type': 'month',
            'planning_date': '2016-02-14',
            'filters': 't2',
        }

        response = self.client.post(url, data)
        self.assertEqual(302, response.status_code)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Profile', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='categorypermission',
            name='can_edit_groups',
            field=models.ManyToManyField(default=None, related_name='can_edit_perm', to='Crm.Group', blank=True),
        ),
        migrations.AlterField(
            model_name='categorypermission',
            name='can_view_groups',
            field=models.ManyToManyField(default=None, related_name='can_view_perm', to='Crm.Group', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='action',
            name='contacts',
            field=models.ManyToManyField(default=None, to='Crm.Contact', verbose_name='contacts', blank=True),
        ),
        migrations.AlterField(
            model_name='action',
            name='entities',
            field=models.ManyToManyField(default=None, to='Crm.Entity', verbose_name='entities', blank=True),
        ),
        migrations.AlterField(
            model_name='actiontype',
            name='allowed_status',
            field=models.ManyToManyField(default=None, help_text='Action of this type allow the given status', to='Crm.ActionStatus', blank=True),
        ),
        migrations.AlterField(
            model_name='city',
            name='groups',
            field=models.ManyToManyField(related_name='city_groups_set', verbose_name='group', to='Crm.Zone', blank=True),
        ),
        migrations.AlterField(
            model_name='contact',
            name='relationships',
            field=models.ManyToManyField(default=None, to='Crm.Contact', through='Crm.Relationship', blank=True),
        ),
        migrations.AlterField(
            model_name='contact',
            name='role',
            field=models.ManyToManyField(default=None, to='Crm.EntityRole', verbose_name='Roles', blank=True),
        ),
        migrations.AlterField(
            model_name='contactsimport',
            name='groups',
            field=models.ManyToManyField(default=None, help_text='The created entities will be added to the selected groups.', verbose_name='groups', to='Crm.Group', blank=True),
        ),
        migrations.AlterField(
            model_name='group',
            name='contacts',
            field=models.ManyToManyField(to='Crm.Contact', blank=True),
        ),
        migrations.AlterField(
            model_name='group',
            name='entities',
            field=models.ManyToManyField(to='Crm.Entity', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""search views and actions"""

from __future__ import unicode_literals

import json
import xlwt

from django.db.models import Q
from django.contrib import messages
from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import user_passes_test
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render
from django.template import Template, Context
from django.utils.translation import ugettext as _

from colorbox.decorators import popup_redirect, popup_close
from coop_cms.models import Newsletter
from coop_cms.utils import paginate
from wkhtmltopdf.views import PDFTemplateView

from balafon.permissions import can_access, is_admin
from balafon.utils import logger, log_error, HttpResponseRedirectMailtoAllowed
from balafon.Crm.models import Entity, Contact, Group, Action, Opportunity, City, CustomField, Subscription
from balafon.Crm import settings as crm_settings
from balafon.Emailing.models import Emailing
from balafon.Emailing.forms import NewEmailingForm
from balafon.Search.models import Search
from balafon.Search.forms import (
    ActionForContactsForm, FieldChoiceForm, GroupForContactsForm, QuickSearchForm, PdfTemplateForm, SearchForm,
    SearchNameForm, get_field_form, SubscribeContactsAdminForm
)


def filter_icontains_unaccent(queryset, field, text):
    if crm_settings.is_unaccent_filter_supported():
        queryset = queryset.extra(
            where=["UPPER(unaccent("+field+")) LIKE UPPER(unaccent(%s))"],
            params=["%{0}%".format(text)]
        )
        return list(queryset)
    return list(queryset.filter(**{field+"__icontains": text}))


@user_passes_test(can_access)
def quick_search(request):
    if request.method == "POST":
        form = QuickSearchForm(request.POST)
        if form.is_valid():
            text = form.cleaned_data["text"]
            
            queryset = Entity.objects.filter(is_single_contact=False)
            entities_by_name = filter_icontains_unaccent(queryset, 'name', text)
            
            queryset = Contact.objects.all()#.filter(has_left=False)
            contacts_by_name = filter_icontains_unaccent(queryset, 'lastname', text)
            
            for entity in entities_by_name:
                setattr(entity, 'is_entity', True)
                for contact in entity.contact_set.all():
                    try:
                        # avoid duplicates
                        contacts_by_name.remove(contact)
                    except ValueError:
                        pass
            contacts = entities_by_name + contacts_by_name
            contacts.sort(key=lambda x: getattr(x, 'name', getattr(x, 'lastname', '')))
            
            queryset = Group.objects.all()
            groups_by_name = filter_icontains_unaccent(queryset, 'name', text)
            
            contacts_by_email = Contact.objects.filter(
                Q(email__icontains=text) | (Q(email="") & Q(entity__email__icontains=text)))
            
            cities_by_name = []
            for city in City.objects.filter(name__icontains=text):
                entities_count, contacts_count = city.contact_set.count(), city.entity_set.count()
                if entities_count + contacts_count:
                    cities_by_name.append((city, entities_count, contacts_count))
            
            contacts_by_phone = list(Contact.objects.filter(Q(mobile__icontains=text) | Q(phone__icontains=text)))
            contacts_by_phone += list(Entity.objects.filter(phone__icontains=text))
            
            context_dict = {
                'contacts_by_phone': contacts_by_phone,
                'cities_by_name': cities_by_name,
                'text': text,
                'contacts_by_email': contacts_by_email,
                'contacts': contacts,
                'groups_by_name': groups_by_name,
            }
                
            return render(
                request,
                'Search/quicksearch_results.html',
                context_dict
            )
    else:
        raise Http404


@user_passes_test(can_access)
def search(request, search_id=0, group_id=0, opportunity_id=0, city_id=0):
    message = ''
    results = []
    search_obj = None
    field_choice_form = FieldChoiceForm()
    contains_refuse_newsletter = False
    data = None
    contacts_count = 0
    has_empty_entities = False
    group = opportunity = city = None
    contacts_display = False

    if request.method == "POST":
        data = request.POST
    elif group_id:
        group = get_object_or_404(Group, id=group_id)
        data = {"gr0-_-group-_-0": group_id}
    elif opportunity_id:
        opportunity = get_object_or_404(Opportunity, id=opportunity_id)
        data = {"gr0-_-opportunity-_-0": opportunity_id}
    elif city_id:
        city = get_object_or_404(City, id=city_id)
        data = {"gr0-_-city-_-0": city_id}
            
    if data:
        search_form = SearchForm(data)
        if search_form.is_valid():
            contacts_display = search_form.contacts_display
            
            if not contacts_display:
                results, contacts_count, has_empty_entities = search_form.get_contacts_by_entity()
                contains_refuse_newsletter = search_form.contains_refuse_newsletter
            else:
                results = search_form.get_contacts()
                contacts_count, has_empty_entities = len(results), False
                contains_refuse_newsletter = search_form.contains_refuse_newsletter
            
            if not results:
                message = _('Sorry, no results found')
    else:
        search_obj = get_object_or_404(Search, id=search_id) if search_id else None
        search_form = SearchForm(instance=search_obj)
    
    entities_count = 0 if contacts_display else len(results)

    page_obj = paginate(request, results, getattr(settings, 'BALAFON_SEARCH_NB_IN_PAGE', None) or 50)

    return render(
        request,
        'Search/search.html',
        {
            'page_obj': page_obj,
            'results': list(page_obj),
            'field_choice_form': field_choice_form,
            'message': message,
            'has_empty_entities': has_empty_entities,
            'search_form': search_form,
            'search': search_obj,
            'contacts_count': contacts_count,
            'entities_count': entities_count,
            'contains_refuse_newsletter': contains_refuse_newsletter,
            'group': group,
            'opportunity': opportunity,
            'city': city,
            'contacts_display': contacts_display,
        }
    )


@user_passes_test(can_access)
@popup_redirect
def save_search(request, search_id):
    try:
        search_id = int(search_id)
        if search_id:
            get_object_or_404(Search, id=search_id)
    except ValueError:
        raise Http404
    
    if request.method == "POST":
        form = SearchNameForm(request.POST)
        if form.is_valid():
            search_obj = form.cleaned_data['name']
            search_fields = form.cleaned_data['search_fields']
            search_fields['name'] = search_obj.name
            search_form = SearchForm(search_fields, instance=search_obj, save=True)
            if search_form.is_valid():
                search_form.save_search()
                return HttpResponseRedirect(reverse('search', args=[search_obj.id]))
    else:
        if search_id:
            search_obj = get_object_or_404(Search, id=search_id)
            initial = {'name': search_obj.name, 'search_id': search_id}
        else:
            initial = {'search_id': 0}
        form = SearchNameForm(initial=initial)
    
    return render(
        request,
        'Search/search_name.html',
        {
            'form': form,
            'search_id': search_id,
        }
    )


@user_passes_test(can_access)
def get_field(request, name):
    block = request.GET.get('block')
    count = request.GET.get('count')
    if not (block and count):
        raise Http404
    try:
        form_class = get_field_form(name)
        the_template = Template('{{form.as_it_is}}')
        the_context = {'form': form_class(block, count)}
        return HttpResponse(
            json.dumps({'form': the_template.render(Context(the_context))}),
            content_type="application/json"
        )
    except KeyError:
        raise Http404
    except Exception as msg:
        logger.exception("get_field")
        raise


@user_passes_test(can_access)
def mailto_contacts(request, bcc):
    """Open the mail client in order to send email to contacts"""
    if request.method == "POST":
        nb_limit = getattr(settings, 'BALAFON_MAILTO_LIMIT', 25) or 25
        search_form = SearchForm(request.POST)
        if search_form.is_valid():
            emails = search_form.get_contacts_emails()
            if emails:
                if len(emails) > nb_limit:
                    if getattr(settings, 'BALAFON_MAILTO_LIMIT_AS_TEXT', False):
                        return HttpResponse(', '.join(emails), content_type='text/plain')
                    else:
                        index_from, email_groups = 0, []
                        nb_emails = len(emails)
                        while True:
                            index_to = index_from + nb_limit
                            if index_to < nb_emails:
                                email_groups.append(emails[index_from:index_to])
                            else:
                                email_groups.append(emails[index_from:])
                                break
                            index_from = index_to
                        return render(
                            request,
                            'Search/mailto_groups.html',
                            {'bcc': int(bcc), 'email_groups': email_groups, 'nb_limit': nb_limit}
                        )
                else:
                    mailto = 'mailto:'
                    if int(bcc): mailto += '?bcc='
                    mailto += ','.join(emails)
                    return HttpResponseRedirectMailtoAllowed(mailto)
            else:
                return HttpResponse(_('Mailto: Error, no emails defined'), content_type='text/plain')
    raise Http404


@user_passes_test(can_access)
def view_search_list(request):
    searches = Search.objects.all()
    page_obj = paginate(request, searches, 50)

    return render(
        request,
        'Search/search_list.html',
        {
            'searches': list(page_obj),
            'page_obj': page_obj,
        }
    )


@user_passes_test(can_access)
@popup_redirect
def create_emailing(request):
    try:
        if request.method == "POST":
            if "create_emailing" in request.POST:
                # called by the colorbox
                form = NewEmailingForm(request.POST)
                if form.is_valid():
                    newsletter_id = form.cleaned_data['newsletter']
                    if newsletter_id:
                        newsletter = Newsletter.objects.get(id=newsletter_id)
                    else:
                        newsletter = Newsletter.objects.create(subject=form.cleaned_data['subject'])

                    subscription_type = form.cleaned_data['subscription_type']

                    contacts = form.get_contacts()

                    emailing = Emailing.objects.create(
                        newsletter=newsletter,
                        subscription_type=subscription_type,
                        lang=form.cleaned_data['lang']
                    )
                    for contact in contacts:
                        emailing.send_to.add(contact)

                    emailing.from_email = form.cleaned_data['from_email']

                    emailing.save()
                    
                    return HttpResponseRedirect(newsletter.get_absolute_url())
                else:
                    return render(
                        request,
                        'Search/create_action_for_contacts.html',
                        {'form': form}
                    )
            else:
                search_form = SearchForm(request.POST)
                if search_form.is_valid():
                    contacts = search_form.get_contacts()
                    form = NewEmailingForm(initial={'contacts': contacts})
                    return render(
                        request,
                        'Search/new_emailing.html',
                        {'form': form}
                    )
    except Exception as msg:
        logger.exception("create_emailing")
        raise
    raise Http404


@user_passes_test(can_access)
def export_contacts_as_excel(request):
    if request.method == "POST":
        search_form = SearchForm(request.POST)
        if search_form.is_valid():
            contacts = search_form.get_contacts()
            if not search_form.contacts_display:
                # if the form already has a sort criteria: don't sort again
                contacts.sort(key=lambda x: "{0}-{1}-{2}".format(x.entity, x.lastname, x.firstname))
            
            # create the excel document
            workbook = xlwt.Workbook()
            worksheet = workbook.add_sheet('balafon')

            fields = [
                'id', 'get_gender_text', 'lastname', 'firstname', 'title', 'get_entity_name', 'job', 'role',
                'get_address', 'get_address2', 'get_address3', 'get_zip_code', 'get_cedex', 'get_city',
                'get_foreign_country', 'mobile', 'get_phone', 'get_email', 'birth_date'
            ]
            
            # header
            header_style = xlwt.easyxf('font: bold 1; pattern: pattern solid, fore-colour gray25;')
            # create a map of verbose name for each field
            field_dict = dict([(field.name, _(field.verbose_name).capitalize()) for field in Contact._meta.fields])
            field_dict['foreign_country'] = _("Country")
            field_dict['entity_name'] = _("Entity")
            
            # Add custom fields
            for cf in CustomField.objects.filter(export_order__gt=0).order_by('export_order'):
                if cf.model == CustomField.MODEL_CONTACT:
                    fields.append('custom_field_'+cf.name)
                    field_dict['custom_field_'+cf.name] = cf.label
                elif cf.model == CustomField.MODEL_ENTITY:
                    fields.append('entity_custom_field_'+cf.name)
                    field_dict['entity_custom_field_'+cf.name] = cf.label
            
            for index, field in enumerate(fields):
                if field[:4] == 'get_':
                    field = field[4:]
                    if field[-8:] == '_display':
                        field = field[:-8]
                # print the verbose name if exists, the field name if not
                value = field_dict.get(field, field)
                worksheet.write(0, index, value, header_style)
            
            style = xlwt.Style.default_style
            for index, contact in enumerate(contacts):
                for index2, field_name in enumerate(fields):
                    field = getattr(contact, field_name)

                    if field_name == 'role':
                        field = ", ".join([r.name for r in field.all()])

                    elif callable(field):
                        field = field()

                    if field:
                        worksheet.write(index+1, index2, '{0}'.format(field), style)

            response = HttpResponse(content_type='application/vnd.ms-excel')
            response['Content-Disposition'] = 'attachment; filename={0}.xls'.format('balafon')
            workbook.save(response)
            return response
        else:
            logger.error('{0}'.format(search_form.errors))
    raise Http404


@user_passes_test(can_access)
@popup_redirect
def create_action_for_contacts(request):
    if request.method == "POST":
        if "create_actions" in request.POST:
            form = ActionForContactsForm(request.POST)
            if form.is_valid():
                contacts = form.get_contacts()
                for contact in contacts:
                    # create actions for each contact
                    kwargs = dict(form.cleaned_data)
                    for key in ('date', 'time', 'contacts'):
                        del kwargs[key]
                    action = Action.objects.create(**kwargs)
                    if action.type and action.type.default_status:
                        action.status = action.type.default_status
                    action.contacts.add(contact)
                    action.save()
                messages.add_message(
                    request,
                    messages.SUCCESS,
                    _("{0} actions have been created".format(len(contacts)))
                )
                return HttpResponseRedirect(reverse('crm_board_panel'))
            else:
                return render(
                    request,
                    'Search/create_action_for_contacts.html',
                    {'form': form}
                )
        else:
            search_form = SearchForm(request.POST)
            if search_form.is_valid():
                contacts = search_form.get_contacts()
                if contacts:
                    form = ActionForContactsForm(initial={'contacts': contacts})
                    return render(
                        request,
                        'Search/create_action_for_contacts.html',
                        {'form': form}
                    )
                else:
                    return render(
                        request,
                        'balafon/message_dialog.html',
                        {
                            'title': _('Create action for contacts'),
                            'message': _('The search results contains no contacts')
                        }
                    )
    raise Http404


@user_passes_test(can_access)
@popup_redirect
def add_contacts_to_group(request):
    try:
        if request.method == "POST":
            if "add_to_group" in request.POST:
                form = GroupForContactsForm(request.POST)
                if form.is_valid():
                    contacts = form.get_contacts()
                    groups = form.cleaned_data['groups']
                    
                    if form.cleaned_data["on_contact"]:
                        for g in groups:
                            for c in contacts:
                                g.contacts.add(c)
                            g.save()
                        messages.add_message(
                            request, messages.SUCCESS, _("{0} contacts have been added to groups".format(len(contacts)))
                        )
                    else:
                        entities = set([c.entity for c in contacts])
                        for g in groups:
                            for entity in entities:
                                g.entities.add(entity)
                            g.save()
                        messages.add_message(
                            request, messages.SUCCESS, _("{0} entities have been added to groups".format(len(entities)))
                        )
                    return HttpResponseRedirect(reverse('crm_board_panel'))
                else:
                    return render(
                        request,
                        'Search/add_contacts_to_group.html',
                        {'form': form}
                    )
            else:
                search_form = SearchForm(request.POST)
                if search_form.is_valid():
                    contacts = search_form.get_contacts()
                    if contacts:
                        form = GroupForContactsForm(initial={'contacts': contacts})
                        return render(
                            request,
                            'Search/add_contacts_to_group.html',
                            {'form': form}
                        )
                    else:
                        return render(
                            request,
                            'balafon/message_dialog.html',
                            {
                                'title': _('Add contacts to group'),
                                'message': _('The search results contains no contacts')
                            }
                        )
    except Exception as msg:
        logger.exception("add_contacts_to_group")
        raise
    raise Http404


@user_passes_test(is_admin)
@popup_close
def subscribe_contacts_admin(request):
    
    if not request.user.is_superuser:
        raise PermissionDenied
    
    try:
        if request.method == "POST":
            if "subscribe_contacts_admin" in request.POST:
                form = SubscribeContactsAdminForm(request.POST)
                if form.is_valid():
                    nb_contacts = 0
                    contacts = form.get_contacts()
                    subscription_type = form.cleaned_data['subscription_type']
                    subscribe = form.cleaned_data['subscribe']

                    for contact in contacts:

                        if subscribe:
                            subscription, created = Subscription.objects.get_or_create(
                                contact=contact, subscription_type=subscription_type
                            )
                        else:
                            created = False
                            try:
                                subscription = Subscription.objects.get(
                                    contact=contact, subscription_type=subscription_type
                                )
                            except Subscription.DoesNotExist:
                                subscription = None

                        if created or (subscription and subscribe != subscription.accept_subscription):
                            subscription.accept_subscription = subscribe
                            subscription.save()
                            nb_contacts += 1

                    if subscribe:
                        messages.add_message(
                            request,
                            messages.SUCCESS,
                            _("{0} contacts have subscribe to the newsletter".format(nb_contacts))
                        )
                    else:
                        messages.add_message(
                            request,
                            messages.SUCCESS,
                            _("{0} contacts have unsubscribe to the newsletter".format(nb_contacts))
                        )

                    return None  # popup_close will return js code to close the popup
                else:
                    return render(
                        request,
                        'Search/subscribe_contacts_admin_form.html',
                        {'form': form}
                    )
            else:
                search_form = SearchForm(request.POST)
                if search_form.is_valid():
                    contacts = search_form.get_contacts()
                    if contacts:
                        form = SubscribeContactsAdminForm(initial={'contacts': contacts})
                        return render(
                            request,
                            'Search/subscribe_contacts_admin_form.html',
                            {'form': form}
                        )
                    else:
                        return render(
                            request,
                            'balafon/message_dialog.html',
                            {
                                'title': _('Subscribe contacts admin'),
                                'message': _('The search results contains no contacts')
                            }
                        )
    except Exception:
        logger.exception("search_subscribe_contacts_admin")
        raise
    raise Http404


@user_passes_test(can_access)
@popup_redirect
@log_error
def export_to_pdf(request):
    try:
        if request.method == "POST":
            if "export_to_pdf" in request.POST:
                #called by the colorbox
                form = PdfTemplateForm(request.POST)
                if form.is_valid():
                    template_name = form.cleaned_data['template']
                    contacts = form.get_contacts()
                    context = {
                        "contacts": contacts,
                        "search_dict": json.loads(form.cleaned_data['search_dict']),
                    }
                    
                    context = form.patch_context(context)
                    
                    pdf_options = getattr(settings, 'BALAFON_PDF_OPTIONS', None)
                    if pdf_options is None:
                        cmd_options = {'margin-top': 0, 'margin-bottom': 0, 'margin-right': 0, 'margin-left': 0, }
                    else:
                        cmd_options = pdf_options.get(template_name, {})
                    
                    pdf_view = PDFTemplateView(
                        filename='balafon.pdf',
                        template_name=template_name,
                        request=request,
                        cmd_options=cmd_options
                    )
                    return pdf_view.render_to_response(context)

                else:
                    return render(
                        request,
                        'Search/export_to_pdf.html',
                        {'form': form}
                    )
            else:
                search_form = SearchForm(request.POST)
                if search_form.is_valid():
                    contacts = search_form.get_contacts()
                    search_dict = json.dumps(search_form.serialize())
                    form = PdfTemplateForm(initial={'contacts': contacts, 'search_dict': search_dict})
                    return render(
                        request,
                        'Search/export_to_pdf.html',
                        {'form': form}
                    )
    except Exception as msg:
        logger.exception("export_to_pdf")
        raise
    raise Http404



# -*- coding: utf-8 -*-
"""urls"""

from __future__ import unicode_literals

from django.conf.urls import url

from balafon.Store.views.sales_documents import (
    SalesDocumentView, SalesDocumentPdfView, SalesDocumentPublicView, SalesDocumentPublicPdfView
)
from balafon.Store.views.statistics import StaticticsIndexView, AddExtraSaleView
from balafon.Store.views.xls_export import (
    StockXlsView, StockAlertXlsView, StoreXlsCatalogueView, StockImportView, StoreItemXlsView, ActionSummaryFormView,
    ActionSummaryXlsView
)


urlpatterns = [
    url(
        r'^view-sales-document/(?P<action_id>\d+)/$',
        SalesDocumentView.as_view(),
        name='store_view_sales_document'
    ),

    url(
        r'^document/(?P<action_uuid>[\w\d-]+)/$',
        SalesDocumentPublicView.as_view(),
        name='store_view_sales_document_public'
    ),

    url(
        r'^view-sales-document/(?P<action_id>\d+)/pdf/$',
        SalesDocumentPdfView.as_view(),
        name='store_view_sales_document_pdf'
    ),

    url(
        r'^document/(?P<action_uuid>[\w\d-]+)/pdf/$',
        SalesDocumentPublicPdfView.as_view(),
        name='store_view_sales_document_pdf_public'
    ),

    url(r"^admin/export-stock/$", StockXlsView.as_view(), name='store_store_item_admin_export'),

    url(
        r"^admin/export-stock-alert/$",
        StockAlertXlsView.as_view(),
        name='store_store_item_admin_export_alert'
    ),
    url(
        r"^admin/store-catalogue/(?P<category_id>\d+)$",
        StoreXlsCatalogueView.as_view(),
        name=r'store_xls_catalogue'
    ),

    url(
        r"^admin/store-articles/(?P<category_id>\d+)/(?P<supplier_id>\d+)/$",
        StoreItemXlsView.as_view(),
        name=r'store_xls_articles'
    ),

    url(
        r'statistics/index/$',
        StaticticsIndexView.as_view(),
        name=r'store_statistics_index'
    ),

    url(
        'statistics/add-extra-sale/$',
        AddExtraSaleView.as_view(),
        name=r'store_add_extra_sale'
    ),

    url(
        r"^admin/import-stock/$",
        StockImportView.as_view(),
        name='store_import_stock'
    ),

    url(
        r"^admin/actions-summary/(?P<action_type_id>\d+)/$",
        ActionSummaryFormView.as_view(),
        name='store_actions_summary'
    ),

    url(
        r"^admin/actions-summary-xls/(?P<action_type_id>\d+)/(?P<start_date>.+)/(?P<end_date>.+)/$",
        ActionSummaryXlsView.as_view(),
        name='store_actions_summary_xls'
    ),

]


# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function

from django.core.management.base import BaseCommand

from balafon.Crm import models


class Command(BaseCommand):
    help = u"add a new credit of emailings"
    use_argparse = False

    def handle(self, *args, **options):
        verbose = options.get('verbosity', 0)

        for region in models.Zone.objects.filter(type__type='region'):
            print(region.name)

        print('**********')

        large_region_type = models.ZoneType.objects.get_or_create(name='Grandes régions', type='large_region')[0]

        mapping = [
            ['Hauts de France', ['Nord-Pas-de-Calais', 'Picardie']],
            ['Normandie', ['Haute-Normandie', 'Basse-Normandie']],
            ['Ile-de-France', ['Ile-de-France']],
            ['Grand-Est', ['Alsace', 'Champagne-Ardenne', 'Lorraine']],
            ['Bretagne', ['Bretagne']],
            ['Pays de la Loire', ['Pays de la Loire']],
            ['Centre Val de Loire', ['Centre']],
            ['Bourgogne Franche-Comté', ['Bourgogne', 'Franche-Comté']],
            ['Nouvelle Aquitaine', ['Aquitaine', 'Poitou-Charentes', 'Limousin']],
            ['Occitanie Pyrénées-Méditerranée', ['Midi-Pyrénées', 'Languedoc-Roussillon']],
            ['Auvergne Rhône-Alpes', ['Auvergne', 'Rhône-Alpes']],
            [u"Provence-Alpes-Côte d'Azur", [u"Provence-Alpes-Côte d'Azur"]],
            [u"Corse", ['Corse']],
            [u"Guyane", ['Guyane']],
        ]

        for large_region_name, child_region_names in mapping:

            large_region = models.Zone.objects.get_or_create(
                name=large_region_name, type=large_region_type
            )[0]

            for child_region_name in child_region_names:
                try:
                    child_region = models.Zone.objects.get(
                        name=child_region_name, type__type='region'
                    )
                    child_region.groups.add(large_region)
                    child_region.save()
                except models.Zone.DoesNotExist:
                    print("#", child_region_name, "missing")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0014_auto_20161108_0940'),
    ]

    operations = [
        migrations.AddField(
            model_name='contact',
            name='gender_title',
            field=models.CharField(default='', help_text='Overwrites gender if defined', max_length=50, verbose_name='gender title', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib import admin

from . import models


admin.site.register(models.ContactProfile)


class CategoryPermissionAdmin(admin.ModelAdmin):
    list_display = ['__str__']
    filter_horizontal = ['can_view_groups', 'can_edit_groups']

admin.site.register(models.CategoryPermission, CategoryPermissionAdmin)



# -*- coding: utf-8 -*-
"""unit testing"""

from __future__ import unicode_literals

from django.conf import settings
from django.core.urlresolvers import reverse
from django.test.utils import override_settings

from coop_cms.tests import BeautifulSoup
from model_mommy import mommy

from balafon.unit_tests import response_as_json
from balafon.Crm import models
from balafon.Crm.tests import BaseTestCase


class ViewEntityTest(BaseTestCase):
    """View an entity"""

    def test_view_entity(self):
        """view entity"""
        entity = mommy.make(models.Entity)
        contact = entity.default_contact
        url = entity.get_absolute_url()
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, entity.name)
        self.assertContains(response, reverse("crm_view_contact", args=[contact.id]))
        self.assertTemplateUsed(response, "balafon/bs_base.html")

    def test_preview_entity(self):
        """view entity in popup"""
        entity = mommy.make(models.Entity)
        contact = entity.default_contact
        url = entity.get_preview_url()
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, entity.name)
        self.assertContains(response, reverse("crm_view_contact", args=[contact.id]))
        self.assertTemplateUsed(response, "balafon/bs_base_raw.html")

    def test_view_entity_secondary_contact(self):
        """view entity with secondary contact"""
        entity = mommy.make(models.Entity)
        entity.default_contact
        contact1 = mommy.make(models.Contact, main_contact=True, entity=entity)
        contact2 = mommy.make(models.Contact, main_contact=False, entity=entity)

        url = reverse('crm_view_entity', args=[entity.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, entity.name)
        url1 = reverse("crm_view_contact", args=[contact1.id])
        url2 = reverse("crm_view_contact", args=[contact2.id])
        self.assertContains(response, url1)
        self.assertContains(response, url2)
        tag1 = BeautifulSoup(response.content).select('.ut-contact-{0.id} .ut-secondary-contact'.format(contact1))
        self.assertEqual(len(tag1), 0)
        tag2 = BeautifulSoup(response.content).select('.ut-contact-{0.id} .ut-secondary-contact'.format(contact2))
        self.assertEqual(len(tag2), 1)

    def test_view_entity_has_left_contact(self):
        """view entity with 'has left' contact"""
        entity = mommy.make(models.Entity)
        contact = entity.default_contact
        contact1 = mommy.make(models.Contact, has_left=False, entity=entity)
        contact2 = mommy.make(models.Contact, has_left=True, entity=entity)

        url = reverse('crm_view_entity', args=[entity.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, entity.name)
        url1 = reverse("crm_view_contact", args=[contact.id])
        url2 = reverse("crm_view_contact", args=[contact2.id])
        self.assertContains(response, url1)
        self.assertContains(response, url2)
        tag1 = BeautifulSoup(response.content).select('.ut-contact-{0.id} .ut-has-left'.format(contact1))
        self.assertEqual(len(tag1), 0)
        tag2 = BeautifulSoup(response.content).select('.ut-contact-{0.id} .ut-has-left'.format(contact2))
        self.assertEqual(len(tag2), 1)


class CreateEntityTest(BaseTestCase):
    """Create entities"""

    def test_view_create_entity_no_type(self):
        """view the create entity page no type"""
        url = reverse('crm_create_entity', args=[0])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Entity.objects.count(), 0)

    def test_view_create_entity_with_type(self):
        """view the create entity page with type"""
        entity_type = mommy.make(models.EntityType)
        url = reverse('crm_create_entity', args=[entity_type.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Entity.objects.count(), 0)

    def test_create_entity_no_type(self):
        """create an entity with no type"""
        url = reverse('crm_create_entity', args=[0])
        response = self.client.post(url, data={'name': "ABC"})
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Entity.objects.count(), 1)
        entity = models.Entity.objects.all()[0]
        self.assertContains(response, 'colorbox')
        self.assertContains(response, reverse("crm_view_entity", args=[entity.id]))
        self.assertEqual(entity.name, "ABC")
        self.assertEqual(entity.type, None)

    def test_create_entity_with_type(self):
        """create enity with type"""
        entity_type = mommy.make(models.EntityType)
        url = reverse('crm_create_entity', args=[entity_type.id])
        response = self.client.post(url, data={'name': "ABC", "type": entity_type.id})
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Entity.objects.count(), 1)
        entity = models.Entity.objects.all()[0]
        self.assertContains(response, 'colorbox')
        self.assertContains(response, reverse("crm_view_entity", args=[entity.id]))
        self.assertEqual(entity.name, "ABC")
        self.assertEqual(entity.type, entity_type)

    def test_create_entity_with_type_after(self):
        """create entity and set type in form"""
        entity_type = mommy.make(models.EntityType)
        url = reverse('crm_create_entity', args=[0])
        response = self.client.post(url, data={'name': "ABC", "type": entity_type.id})
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Entity.objects.count(), 1)
        entity = models.Entity.objects.all()[0]
        self.assertContains(response, 'colorbox')
        self.assertContains(response, reverse("crm_view_entity", args=[entity.id]))
        self.assertEqual(entity.name, "ABC")
        self.assertEqual(entity.type, entity_type)

    def test_create_entity_url(self):
        """create entity with website value"""
        url = reverse('crm_create_entity', args=[0])
        response = self.client.post(url, data={'name': "ABC", "website": "http://toto.fr/"})
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Entity.objects.count(), 1)
        entity = models.Entity.objects.all()[0]
        self.assertContains(response, 'colorbox')
        self.assertContains(response, reverse("crm_view_entity", args=[entity.id]))
        self.assertEqual(entity.name, "ABC")
        self.assertEqual(entity.website, "http://toto.fr/")

    def test_create_entity_url_no_scheme(self):
        """create an entity: no http:// in website value"""
        url = reverse('crm_create_entity', args=[0])
        response = self.client.post(url, data={'name': "ABC", "website": "toto.fr/"})
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Entity.objects.count(), 1)
        entity = models.Entity.objects.all()[0]
        self.assertContains(response, 'colorbox')
        self.assertContains(response, reverse("crm_view_entity", args=[entity.id]))
        self.assertEqual(entity.name, "ABC")
        self.assertEqual(entity.website, "http://toto.fr/")

    def test_view_create_entity_unknown_type(self):
        """view the create entity page with an unknown type id"""
        entity_type = mommy.make(models.EntityType)
        url = reverse('crm_create_entity', args=[2222])
        response = self.client.get(url)
        self.assertEqual(404, response.status_code)
        self.assertEqual(models.Entity.objects.count(), 0)

        response = self.client.post(url, data={'name': "ABC", "type": 2222})
        self.assertEqual(404, response.status_code)
        self.assertEqual(models.Entity.objects.count(), 0)

        url = reverse('crm_create_entity', args=[entity_type.id])
        response = self.client.post(url, data={'name': "ABC", "type": "2222"})
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Entity.objects.count(), 0)

    def test_view_create_entity_invalid_type(self):
        """view the create entity type with an invalid value"""
        entity_type = mommy.make(models.EntityType)
        url = reverse('crm_create_entity', args=[2222]).replace("2222", "aaa")
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)
        self.assertEqual(models.Entity.objects.count(), 0)

        response = self.client.post(url, data={'name': "ABC", "type": 2222})
        self.assertTrue(response.status_code in [404, 405])
        self.assertEqual(models.Entity.objects.count(), 0)

        url = reverse('crm_create_entity', args=[entity_type.id])
        response = self.client.post(url, data={'name': "ABC", "type": "aaaaa"})
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Entity.objects.count(), 0)


class EditEntityTestCase(BaseTestCase):
    """edit entity"""

    fixtures = ['zones.json', ]

    def _check_redirect_url(self, response, next_url):
        """check redirect url"""
        redirect_url = response.redirect_chain[-1][0]
        self.assertEqual(redirect_url, "http://testserver"+next_url)

    def test_view_edit_entity(self):
        """view"""
        entity = mommy.make(models.Entity, is_single_contact=False)
        response = self.client.get(reverse('crm_edit_entity', args=[entity.id]))
        self.assertEqual(200, response.status_code)

    @override_settings(BALAFON_SHOW_BILLING_ADDRESS=True)
    def test_view_edit_show_billing_address(self):
        """view edit contact form with billing address setting On"""
        entity = mommy.make(models.Entity, is_single_contact=False)
        response = self.client.get(reverse('crm_edit_entity', args=[entity.id]))
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        self.assertEqual(1, len(soup.select("#id_billing_address")))

    @override_settings(BALAFON_SHOW_BILLING_ADDRESS=False)
    def test_view_edit_hide_billing_address(self):
        """view edit contact form with billing address setting Off"""
        entity = mommy.make(models.Entity, is_single_contact=False)
        response = self.client.get(reverse('crm_edit_entity', args=[entity.id]))
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        self.assertEqual(0, len(soup.select("#id_billing_address")))

    @override_settings()
    def test_default_for_show_billing_address(self):
        """Check default value for BALAFON_SHOW_BILLING_ADDRESS"""
        del settings.BALAFON_SHOW_BILLING_ADDRESS
        entity = mommy.make(models.Entity, is_single_contact=False)
        response = self.client.get(reverse('crm_edit_entity', args=[entity.id]))
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        self.assertEqual(1, len(soup.select("#id_billing_address")))

    def test_edit_entity(self):
        """edit entity"""
        entity = mommy.make(models.Entity, is_single_contact=False)
        url = reverse('crm_edit_entity', args=[entity.id])
        data = {
            'name': 'Dupond SA',
            'city': models.City.objects.get(name="Paris").id,
        }
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        errors = BeautifulSoup(response.content).select('.field-error')
        self.assertEqual(len(errors), 0)
        next_url = reverse('crm_view_entity', args=[entity.id])
        self.assertContains(response, "<script>")
        self.assertContains(response, next_url)

        entity = models.Entity.objects.get(id=entity.id)
        self.assertEqual(entity.name, data['name'])
        self.assertEqual(entity.city.id, data['city'])

    def test_edit_entity_anonymous(self):
        """it should not change if not logged"""
        self.client.logout()

        entity = mommy.make(models.Entity, is_single_contact=False)
        url = reverse('crm_edit_entity', args=[entity.id])
        data = {
            'name': 'Dupond SA',
        }
        response = self.client.post(url, data=data)
        self.assertEqual(302, response.status_code)
        self.assertEqual(302, response.status_code)
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

        entity = models.Entity.objects.get(id=entity.id)
        self.assertNotEqual(entity.name, data['name'])

    def test_edit_entity_not_allowed(self):
        """it should not change if not allowed"""
        self.user.is_staff = False
        self.user.save()

        entity = mommy.make(models.Entity, is_single_contact=False)
        url = reverse('crm_edit_entity', args=[entity.id])
        data = {
            'name': 'Dupond SA',
        }
        response = self.client.post(url, data=data)
        self.assertEqual(302, response.status_code)
        self.assertEqual(302, response.status_code)
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

        entity = models.Entity.objects.get(id=entity.id)
        self.assertNotEqual(entity.name, data['name'])

    def test_edit_entity_keep_notes(self):
        """test misisng fields nor changed"""
        entity = mommy.make(models.Entity, is_single_contact=False, notes="Toto")
        url = reverse('crm_edit_entity', args=[entity.id])
        data = {
            'name': 'Dupond SA',
        }
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        errors = BeautifulSoup(response.content).select('.field-error')
        self.assertEqual(len(errors), 0)
        next_url = reverse('crm_view_entity', args=[entity.id])
        self.assertContains(response, "<script>")
        self.assertContains(response, next_url)

        entity = models.Entity.objects.get(id=entity.id)
        self.assertEqual(entity.name, data['name'])
        self.assertEqual(entity.notes, 'Toto')


class GetEntityIdTestCase(BaseTestCase):
    """edit entity"""

    def test_get_entity_no_name(self):
        """raise Http404"""
        entity = mommy.make(models.Entity, is_single_contact=False, name='ABC')
        response = self.client.get(reverse('crm_get_entity_id'))
        self.assertEqual(404, response.status_code)

    def test_get_entity_name(self):
        """return id"""
        entity1 = mommy.make(models.Entity, is_single_contact=False, name='ABC')
        mommy.make(models.Entity, is_single_contact=False, name='WABC')
        response = self.client.get(reverse('crm_get_entity_id') + '?name={0}'.format(entity1.name))
        self.assertEqual(200, response.status_code)
        data = response_as_json(response)
        self.assertEqual(data['id'], entity1.id)

    def test_get_entity_missing_name(self):
        """raise Http404"""
        entity1 = mommy.make(models.Entity, is_single_contact=False, name='ABC')
        mommy.make(models.Entity, is_single_contact=False, name='WABC')
        response = self.client.get(reverse('crm_get_entity_id') + '?name={0}'.format(entity1.name + 'D'))
        self.assertEqual(404, response.status_code)

    def test_get_duplicate_entity_name(self):
        """raise Http404"""
        entity1 = mommy.make(models.Entity, is_single_contact=False, name='ABC')
        mommy.make(models.Entity, is_single_contact=False, name='ABC')
        response = self.client.get(reverse('crm_get_entity_id') + '?name={0}'.format(entity1.name))
        self.assertEqual(404, response.status_code)




# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-09 13:03
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0023_action_frozen'),
    ]

    operations = [
        migrations.AddField(
            model_name='actionstatus',
            name='next_on_send',
            field=models.ForeignKey(blank=True, default=None, help_text='automatically change the status when sent by email', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='Crm.ActionStatus', verbose_name='Next status on send by email'),
        ),
    ]



# -*- coding: utf-8 -*-
"""notify_due_actions command"""

from __future__ import unicode_literals, print_function

from datetime import date, timedelta

from django.conf import settings
from django.contrib.auth.models import User
from django.core.management.base import BaseCommand
from django.core.mail import EmailMessage
from django.template.loader import get_template
from django.utils.translation import ugettext as _

from balafon.utils import logger


def notify_due_actions(user, actions):
    """send message by email"""

    notification_email = user.email
    from_email = settings.DEFAULT_FROM_EMAIL
        
    data = {
        'user': user,
        'actions': actions,
        'site': settings.COOP_CMS_SITE_PREFIX,
    }
    template = get_template('Users/due_actions_notification_email.txt')
    content = template.render(data)
    
    email = EmailMessage(
        _("Balafon: You have due actions"),
        content,
        from_email,
        [notification_email]
    )
    
    try:
        email.send()
    except Exception:  #pylint: disable=broad-except
        logger.exception("notify_due_actions")


class Command(BaseCommand):
    """The command"""
    help = "notify due actions"
    use_argparse = False

    def handle(self, *args, **options):
        """main of command"""

        verbose = options.get('verbosity', 0)
        tomorrow = date.today() + timedelta(1)

        for user in User.objects.filter(userpreferences__notify_due_actions=True, teammember__active=True):
            
            actions = user.teammember.action_set.filter(planned_date__lt=tomorrow, done=False).order_by("planned_date")
            if actions.count() == 0:
                if verbose:
                    print("no due actions for", user.username)
            else:
                if verbose:
                    print("send", actions.count(), "due actions to", user.username)

                notify_due_actions(user, actions)
            
        if verbose:
            print("done")
